/**
******************************************************************************
* @file    IRPyro_API_single_device_layer.h
* @author  _KEMET Ltd.
* @date    March 2018
* @version Release 1.0.6
* @copyright (c) 2018 _KEMET, Ltd
* @brief   IRPyro data structures and prototype functions
* @verbatim
 ===============================================================================
                      ##### How to use this driver #####
 ===============================================================================
      [..]
       (#) Declare an IRPyro_sensor_device_type in your code
       (#) Fill the values for CS, INT and address
       (#) Fill the values for number_of_active_channels and read_mode
       (#) AFE, CCP and WUP can be left empty if desired, configuration macros
           will fill them with default values
       (#) Call IRPyro_open() to commit the values to the physical IRPyro
       (#) IRPyro_read() retrieves the values sampled by the IRPyro sensor
       (#) Modify the IRPyro settings using IRPyro_IOCTL()
       (#) Close the device when finished using IRPyro_read()
      [..]
   @endverbatim
******************************************************************************
   @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>
   @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.

    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim
******************************************************************************
  */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef IRPyro_API_single_device_layer
#define IRPyro_API_single_device_layer

/* Includes ------------------------------------------------------------------*/
#include <stdint.h>
#include "Micro_controller_template.h"

/** @defgroup API_G6_IRPyro IRPyro AFE values
  * @ingroup IRPyro_ctes
  * @brief Defines to configure the IRPyro AFE register
  * @verbatim
  *    The analogue front end packet contains the settings that control the
  *    common analogue front end settings using 2 Bytes.
  *    More meaningful defines for configuration.
  * @endverbatim
  * @note Do not modify
  * @{
  */
#define INT_DISABLE        0x00
#define INT_ENABLE         0x01
#define TEMP_DISABLE       0x00
#define TEMP_ENABLE        0x01
#define MASTER             0x00
#define SLAVE              0x01
#define CLK_DISABLE        0x00
#define CLK_ENABLE         0x01
#define HP_DISABLE         0x00
#define HP_ENABLE          0x01
#define LOW_POWER_DISABLE  0x00
#define LOW_POWER_ENABLE   0x01
#define LOW_PASS_180       0x00
#define LOW_PASS_90        0x01
#define LOW_PASS_45        0x02
#define LOW_PASS_22_5      0x03
#define LOW_PASS_30        0x00
#define LOW_PASS_15        0x01
#define LOW_PASS_75        0x02
#define LOW_PASS_3_75      0x03

/**
  * @}
  */

/** @defgroup API_G7_IRPyro IRPyro CCP values
  * @ingroup IRPyro_ctes
  * @brief Defines to configure the IRPyro CCP register
  * @verbatim
  *    The channel control packet contains 5 Bytes, each of which corresponds to
  *    a single channel with its individual settings.
  *    More meaningful defines for configuration.
  * @endverbatim
  * @note Do not modify
  * @{
  */
#define CH_DISABLE         0x00
#define CH_ENABLE          0x01
#define FEEDBACK_50_20     0x00
#define FEEDBACK_100_40    0x01
#define FEEDBACK_200_80    0x02
#define FEEDBACK_400_160   0x03
#define FEEDBACK_800_320   0x04
#define FEEDBACK_1600_640  0x05
#define FEEDBACK_3200_1280 0x07
#define GAIN_64            0x00
#define GAIN_32            0x01
#define GAIN_16            0x02
#define GAIN_08            0x03
#define GAIN_04            0x04
#define GAIN_02            0x05
#define GAIN_01            0x07

#define HIGH_PASS_1HZ      0x00
#define HIGH_PASS_2HZ      0x01
#define HIGH_PASS_4HZ      0x02
#define HIGH_PASS_8HZ      0x03
#define HIGH_PASS_0_17HZ   0x00
#define HIGH_PASS_0_33HZ   0x01
#define HIGH_PASS_0_66HZ   0x02
#define HIGH_PASS_1_30HZ   0x03

#define TCONDUCT_1_2T      0x00
#define TCONDUCT_0_6T      0x01
#define TCONDUCT_0_3T      0x02
#define TCONDUCT_0_15T     0x03
/**
  * @}
  */

/** @defgroup API_G8_IRPyro IRPyro WUP values
  * @ingroup IRPyro_ctes
  * @brief Defines to configure the IRPyro WUP register
  * @verbatim
  *    The wake up packet contains the thresholds used in the wake up algorithm.
  *
  *    More meaningful defines for configuration.
  * @endverbatim
  * @note Do not modify
  * @{
  */
#define REF_CH_0           0x00
#define REF_CH_1           0x01
#define REF_CH_2           0x02
#define REF_CH_3           0x03
#define REF_CH_4           0x04
#define SEL_CH_0           0x00
#define SEL_CH_1           0x01
#define SEL_CH_2           0x02
#define SEL_CH_3           0x03
#define SEL_CH_4           0x04
#define SLEEP_MODE_ONE_CH  0x00
#define SLEEP_MODE_TWO_CH  0x01
#define DEFAULT_NO_EVENTS  0x0A
#define NOT_IN_USE_VALUE   0x00
#define MAX_WUP_THRESHOLD  0xFF
#define MIN_WUP_THRESHOLD  0x00
#define SAFE_WUP_THRESHOLD 0xB4
/**
  * @}
  */
/** @defgroup IRPyro_ctes Constants to access IRPyro
  * @ingroup IRPyro_single
  * @brief Definitions used by the IRPyro_sensor_device_type
  * @{
  */
/* Exported constants ------------------------------------------------------------*/
#define IRPyro_CHANNEL_DATA_VALUES      6   // 5 channel values + frame counter
#define IRPyro_CCP_NUMBER_OF_REGISTERS  5   // one CPP register per channel
#define IRPyro_DATA_SIZE               17   // bytes in an IRPyro frame
/** end of group IRPyro_ctes
  * @}
  */

/** @defgroup IRPyro_structs Structures to access IRPyro
  * @ingroup IRPyro_single
  * @brief Collection of structs that define an IRPyro_sensor_device_type
  * @{
  */
/* Exported types ------------------------------------------------------------*/
/** @struct AFE_reg IRPyro_API_single_device_layer.h
  * @brief  IRPyro Analog Front End (AFE) register structure definition
  */
struct AFE_reg
{
    uint8_t  S7_S0;   ///< Sampling rate (8 bit) :Sampling Rate = 1000/(N+1).
    uint8_t  LP;      ///< Enable low power mode: 1 Enable, 0 Disable
    uint8_t  HP;      ///< Enable high pass filter: 1 Enable, 0 Disable.   this is the enable trigger which needs to be set after the filters have been set.
    uint8_t  C_LP;    ///< Low pass selection
    uint8_t  CLK_OUT; ///< 1 enable internal clock output on Clk pin, 0 disable internal clock output on the Clk pin (only apply for Master. For Slave, this is ignored, because the Clk pin is used as input).
    uint8_t  SYNC;    ///< Sync & Clk Pin option: 0 master, 1 slave.
    uint8_t  TEMP;    ///< 0 Normal channel data, 1 temperature data (apply to channel 0 only). When this bit is set, channel 0 will be activated automatically (only apply at normal operation and not in Sleep mode).
    uint8_t  INT;     ///< Interrupt output: 1 Enable, 0 Disable (only apply at normal operation and not in Sleep mode).
};
/** @typedef struct AFE_reg AFE_reg_type;
  * @brief  IRPyro Analog Front End (AFE) register type definition
  */
typedef struct AFE_reg AFE_reg_type;

/** @struct CCP_reg IRPyro_API_single_device_layer.h
  * @brief  IRPyro Channel Control Package register structure definition
  */
struct CCP_reg
{
    // The channel control packet contains 5 Bytes, each of which corresponds to a single channel with its individual settings.
    uint8_t Feedback_cap;              ///< Feedback capacitor selection
    uint8_t High_pass_filter_setting;  ///< High pass selection
    uint8_t Feedback_transconductance; ///< Charge amp. trans-conductance selection
    uint8_t status_bit;                ///< Status bit : Active (1) / Disable (0)
};
/** @typedef struct CCP_reg CCP_reg_type;
  * @brief  IRPyro Channel Control Package register typedef definition
  */
typedef struct CCP_reg CCP_reg_type;

/** @struct WUP_reg IRPyro_API_single_device_layer.h
  * @brief IRPyro Wake Up Package register structure definition
  * @note  probably give these more meaningful names and then add the data sheet descriptor
  */
struct WUP_reg
{
    // The wake up packet contains the thresholds used in the wake up algorithm
    uint8_t UHT;          ///< Byte 0 to 3 (Upper and Lower thresholds) are the 8 MSB of channel ADC data (unsigned value).
    uint8_t ULT;
    uint8_t LHT;
    uint8_t LLT;
    uint8_t WT_Threshold; ///< Byte 4 is the number of samples required to trigger the wake up algorithm. ? UHT and ULT to detect positive wake-up event (UHT > ULT) ? LHT and LLT to detect negative wake-up event (LHT > LLT) ? Number of sample where the signal is in between threshold (WT).
    uint8_t DP0_DP2;      ///< Reference channel coding DP0-DP2: The Reference channel coding, and CH0-CH2: the selected channel coding
    // 000 : select channel 0
    // 001 : select channel 1
    // 010 : select channel 2
    // 011 : select channel 3
    // 1xx : select channel 4
    uint8_t CH0_CH2;      ///< Selected channel coding
    uint8_t ST;           ///< Sleep_mode_type
    // ST: Sleep mode type
    //  0 : one channel sleep type, set by ch0-ch2
    //  1 : two channel sleep type, set by both dp0-dp2 and ch0-ch2
};
/** @typedef struct WUP_reg WUP_reg_type;
  * @brief  IRPyro Wake Up Package register typedef definition
  */
typedef struct WUP_reg WUP_reg_type;

/** @struct FIFO_reg IRPyro_API_single_device_layer.h
  * @brief  IRPyro FIFO packet structure definition
  */
struct FIFO_reg
{
    uint8_t WAKE;        ///<
    uint8_t STATUS_CODE;   ///< response to read operation: 00 no error, 01 fifo full, 10 i2c master read less, 11 i2c master read more
    uint8_t COUNT;       ///< Number of frames available
    uint8_t DATA_READY;  ///< Data ready
};
/** @typedef struct FIFO_reg FIFO_reg_type;
  * @brief  IRPyro FIFO Package register typedef definition
  */
typedef struct FIFO_reg FIFO_reg_type;

/** @struct CCP_filter IRPyro_API_single_device_layer.h
  * @brief  IRPyro Channel filter configuration structure
  */
struct CCP_filter
{
    uint8_t High_pass_filter_setting;  ///< High pass selection
};
/** @typedef struct CCP_filter CCP_filter_type;
  * @brief  Typedef for filter channel definition
  */
typedef struct CCP_filter CCP_filter_type;

/** @struct power_settings IRPyro_API_single_device_layer.h
    * @ingroup IRPyro_structs
  * @brief  IRPyro Power Settings structure definition
  */
struct power_settings
{
    //AFE reg
    uint8_t S7_S0;  ///< sample rate value
    uint8_t LP;     ///< Enable low power mode: 1 Enable, 0 Disable
    uint8_t C_LP;   ///< Low pass filter selection
    //CPP reg
    CCP_filter_type CCP_High_pass[IRPyro_CCP_NUMBER_OF_REGISTERS]; ///< High pass filter configuration for each channel on the IRPyro device
    //WUP reg
    uint8_t UHT; // probably give these more meaningful names and then add the data sheet descriptor
    uint8_t ULT;
    uint8_t LHT;
    uint8_t LLT;
    uint8_t WT_Threshold; //number of samples
};
/** @typedef struct power_settings power_settings_type;
  * @brief
  */
typedef struct power_settings power_settings_type;

/** @struct IRPyro_sensor_device IRPyro_API_single_device_layer.h
 *  @brief  IRPyro device structure definition
 *
 *    Sensor data sampling into the FIFO (SECTION 12.6.1 )
 *    There are two methods of sampling the data available in the FIFO:
 *    Polling mode or Interrupt mode.
 *    - In interrupt mode the host can wait till the interrupt line goes
 *      low in order to read data from the FIFO. The interrupt remains
 *      low till the FIFO is cleared from data.
 *    - In polling mode the host has to read the FIFO status packet to
 *      determine if data is available in order to read it. The number of
 *      data packets available can also be determined from the status
 *      packet allowing multiple reads without having to keep reading the
 *      status packet.
 *    Interrupt mode is a single step operation, polling mode is a two step
 *    operation.
 *
 *     IMPORTANT:
 *       FIFO data collected is stored in the first available position of the
 *       channel_value array.
 *
 *       One channel active:
 *            channel 2 -> channel_value[0]
 *       Four channels active:
 *            channel 1 -> channel_value[0]
 *            channel 2 -> channel_value[1]
 *            channel 3 -> channel_value[2]
 *            channel 4 -> channel_value[3]
 *       For both cases:
 *            frame counter -> channel_value[5]
 */
struct IRPyro_sensor_device
{
    uint8_t      CS_Port;                                      ///<  CS (Chip select) port
    uint8_t      INT_Port;                                     ///< INT (Interrupt / Data Ready ) port
    uint8_t      CS_Pin;                                       ///< CS  pin
    uint8_t      INT_Pin;                                      ///< INT pin
    uint8_t      address;                                      ///< I2C  address in 7 bit format

    uint8_t      number_of_active_channels;                    ///< useful for read()
    uint8_t      read_mode;                                    ///< interrupt (0), polled (1) or synchronized sampling (2)

    AFE_reg_type AFE_register;                                 ///< AFE (analog front end) configuration register
    CCP_reg_type CCP_register[IRPyro_CCP_NUMBER_OF_REGISTERS]; ///< Channel configuration registers 0 - 4
    WUP_reg_type WUP_register;                                 ///< WUP  (Wake Up) configuration register

    FIFO_reg_type FIFO_register;                               ///< FIFO status register

    power_settings_type normal_power;                          ///< normal power configuration
    power_settings_type low_power;                             ///< low power configuration

    uint32_t channel_value[IRPyro_CHANNEL_DATA_VALUES];        ///< Active frame decoded channel data
    uint8_t channel_saturation_byte;                           ///< Active frame saturation byte
    uint8_t raw_data[IRPyro_DATA_SIZE];

};
/** @typedef struct IRPyro_sensor_device IRPyro_sensor_device_type;
  * @brief
  */
typedef struct IRPyro_sensor_device IRPyro_sensor_device_type;

/**@brief Used by the IOCTL function to act a command that modifies AFE and CCP
  */
typedef enum IRPyro_command
{
    sampling_rate_set,
    sampling_rate_get,
    high_pass_set,
    high_pass_get,
    low_pass_set,
    low_pass_get,
    gain_set,
    gain_get,
    power_off,
    power_normal,
    power_low,
    power_sleep,
    power_wakeup,
    register_set,
    register_get
} IRPyro_command_type;

/** @struct IRPyro_cmd IRPyro_API_single_device_layer.h
  * @brief  IRPyro commands
  */
typedef struct IRPyro_cmd
{
    uint8_t cmd_return_code;   ///< command result
    AFE_reg_type AFE_register; ///< AFE (analog front end) configuration register
    CCP_reg_type CCP_register[IRPyro_CCP_NUMBER_OF_REGISTERS]; ///< Channel configuration register
    WUP_reg_type WUP_register; ///< WUP  (Wake Up) configuration register
    IRPyro_command_type cmd;   ///< command
} IRPyro_cmd_type;

/** @struct IRPyro_arg IRPyro_API_single_device_layer.h
  * @brief  IRPyro command argument
  */
typedef struct IRPyro_arg
{
    uint8_t arg;     ///< argument
    uint8_t channel; ///< selected channel
} IRPyro_arg_type;

/** end of group IRPyro_structs
  * @}
  */
/* Exported functions ------------------------------------------------------- */
uint8_t IRPyro_open(IRPyro_sensor_device_type* this_device);
uint8_t IRPyro_close(IRPyro_sensor_device_type* this_device);
uint8_t IRPyro_read(IRPyro_sensor_device_type* this_device);
uint8_t IRPyro_ioctl(IRPyro_sensor_device_type* this_device, IRPyro_cmd_type *cmd, IRPyro_arg_type arg);
/*----------------------------------------------------------------------------*/
#endif

/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
