/**
******************************************************************************
* @file    IRPyro_API_single_device_layer.c
* @author  _KEMET Ltd.
* @date    March 2018
* @version Release 1.0.6
* @copyright (c) 2018 _KEMET, Ltd
* @brief   IRPyro Application interface functions implementation
* @verbatim
===============================================================================
                     ##### How to use this driver #####
===============================================================================
     [..]
       (#) Declare an IRPyro_sensor_device_type in your code
       (#) Fill the values for CS, INT and address
       (#) Fill the values for number_of_active_channels and read_mode
       (#) AFE, CCP and WUP can be left empty if desired, configuration macros
           will fill them with default values
       (#) Call IRPyro_open() to commit the values to the physical IRPyro
       (#) IRPyro_read() retrieves the values sampled by the IRPyro sensor
       (#) Modify the IRPyro settings using IRPyro_IOCTL()
       (#) Close the device when finished using IRPyro_read()
     [..]
  @endverbatim
******************************************************************************
   @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>

   @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.

    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim
******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "Version.h"
#include "IRPyro_API_single_device_layer.h"
/** @defgroup IRPyro_API IRPyro application programming interface
  * @brief Functions to configure and sample data from an IRPyro sensor
  * @{
  */
/** @defgroup IRPyro_single IRPyro Single Device Layer
  * @brief Functions to configure and sample data from a single device
  * @{
  */
/** @defgroup API_G1_IRPyro  IRPyro Constants
 *  @brief Hexadecimal constants for the IRPyro API
 *  @verbatim
 *     Values for the IRPyro commands, memory sizes, configuration values
 *  @endverbatim
 *  @note Do not modify the values since they are hardware dependent
 *  @{
 */

/** @defgroup API_G2_IRPyro IRPyro I2C Commands
  * @brief I2C bytes to control IRPyro
  * @verbatim
  *    One byte constants to configure and control the IRPyro sensor
  * @endverbatim
  * @note Do not modify the values since they are hardware dependent
  * @{
  */

/* list of i2c commands available */
#define IRPyro_TEST             0x00
#define IRPyro_VERSION          0x02
#define IRPyro_FIFO_STATUS      0x04
#define IRPyro_FIFO_READ_FULL   0x06
#define IRPyro_FIFO_READ_ACTIVE 0x08
#define IRPyro_FIFO_CLEAR       0x0A
#define IRPyro_FIFO_RESET       0x0C
#define IRPyro_CCP_READ         0x0E
#define IRPyro_CCP_WRITE        0x10
#define IRPyro_AFE_READ         0x12
#define IRPyro_AFE_WRITE        0x14
#define IRPyro_WUP_READ         0x16
#define IRPyro_WUP_WRITE        0x18
#define IRPyro_ADDR_WRITE       0x1E
#define IRPyro_GO_TO_SLEEP      0x20
#define IRPyro_WAKE_UP          0x22
#define IRPyro_RESET_SOFT       0x24
#define IRPyro_RESET_FULL       0x26

/** end of IRPyro I2C Commands
  * @}
  */
/** @defgroup API_G9_IRPyro IRPyro I2C Operation result codes
  * @brief OK / ERR return codes for IRPyro I2C commands
  * @note Do not modify the values since they are hardware dependent
  * @{
  */
#define IRPyro_TEST_OK           0x01
#define IRPyro_TEST_ERR          0x02
#define IRPyro_FIFO_CLEAR_OK     0X29
#define IRPyro_FIFO_CLEAR_ERR    0X2A
#define IRPyro_FIFO_RESET_OK     0X31
#define IRPyro_FIFO_RESET_ERR    0X32
#define IRPyro_GO_TO_SLEEP_OK    0X81
#define IRPyro_GO_TO_SLEEP_ERR   0X82
#define IRPyro_WAKE_UP_OK        0X89
#define IRPyro_WAKE_UP_ERR       0X8A
#define IRPyro_RESET_SOFT_OK     0X91
#define IRPyro_RESET_SOFT_ERR    0X92
#define IRPyro_RESET_FULL_OK     0X99
#define IRPyro_RESET_FULL_ERR    0X9A

/** end of IRPyro I2C Operation result codes
  * @}
  */

/** @defgroup API_G3_IRPyro IRPyro I2C Default Address
  * @brief Default I2C address on power up
  * @verbatim
  *    IRPyro assumes this value on CS enabled
  *
  * @endverbatim
  * @note Do not modify this value since is hardware dependent
  * @{
  */
#define IRPyro_DEFAULT_I2C     0x65
/**
  * @}
  */

/** @defgroup API_G4_IRPyro IRPyro registers allocation space
  * @brief IRPyro registers sizes in bytes
  * @verbatim
  *    IRPyro sizes used for I2C communications with the MCU
  *
  * @endverbatim
  * @note Do not modify this values since they are hardware dependent
  * @{
  */
#define IRPyro_DATA_SIZE        17      // bytes in an IRPyro frame
#define IRPyro_AFE_SIZE          2      // bytes on the IRPyro AFE register
#define IRPyro_CCP_SIZE          5      // bytes on the IRPyro CCP register
#define IRPyro_WUP_SIZE          6      // bytes on the IRPyro WUP register
#define IRPyro_FIFO_STATUS_SIZE        1  // byte of the IRPyro STS register
#define IRPyro_BYTES_PER_CHANNEL       3  // 24 bits unsigned  channel data MSB to LSB 
#define IRPyro_BYTES_FRAME_COUNT       2  // 16 bit frame counter
#define IRPyro_FIFO_NO_FRAMES         14  // frames of IRPyro_DATA_SIZE
#define IRPyro_MAX_FRAME_COUNT     0xFFFF // maximum value on the frame counter
#define IRPyro_FIFO_WAKE_MASK        0X80 // masks the FIFO WAKE detection from the FIFO status
#define IRPyro_FIFO_INT_MASK         0X01 // masks the FIFO INT from the FIFO status
#define IRPyro_FIFO_COUNT_MASK       0X1E // masks the FIFO count from the FIFO status
#define IRPyro_FIFO_ERROR_MASK       0X60 // masks the FIFO ERROR from the FIFO status
#define IRPyro_FIFO_ERROR_OK         0X00 // FIFO status OK
#define IRPyro_FIFO_ERROR_WRITE_FULL 0X01 // write when FIFO is full (FIFO count = 14) or read when FIFO is empty (FIFO count = 0)
#define IRPyro_FIFO_ERROR_I2C_EARLY  0X02 // detect I2C read FIFO early termination (read less bytes than expected)
#define IRPyro_FIFO_ERROR_I2C_EXTRA  0X03 // detect I2C read FIFO extra (read more bytes than expected)

#define IRPyro_FRAME_COUNT_TOP_BYTE        2 // top byte index location frame count 
#define IRPyro_FRAME_COUNT_LOW_BYTE        1 // low byte index location frame count
#define IRPyro_SHIFT_TO_MSB                8 // bits to shift left the top byte of the frame count
#define IRPyro_SHIFT_ONE                   1 // shift one bit
#define IRPyro_SHIFT_FOUR                  4 // shift FOUR bitS
#define IRPyro_CHANNEL_DATA         0x7FFFFF // 23 bits unsigned channel data
#define IRPyro_CHANNEL_SAT          0X800000 // bit 24 saturation OFF(0)/ON(1)
#define IRPyro_FRAME_COUNT_LOCATION        5 // position on the user data buffer
/**
  * @}
  */

/** API_G1_IRPyro  IRPyro Constants
  * @}
  */

/* Initialization Macros ---------------------------------------------------- */
/*
    .AFE_register.S7_S0   = 0,                 // Sample rate 1 ms
    .AFE_register.LP      = LOW_POWER_DISABLE, // Low power disabled
    .AFE_register.HP      = HP_DISABLE,        // High pass filter disabled
    .AFE_register.C_LP    = LOW_PASS_180,      // Low pass filter selected to 180 Hz
    .AFE_register.CLK_OUT = CLK_ENABLE,        // Internal clock output ENABLED
    .AFE_register.SYNC    = MASTER,            // Device set to master
    .AFE_register.TEMP    = TEMP_DISABLE,      // Temperature detection off
    .AFE_register.INT     = INT_ENABLE,        // Asserted LOW when data ready

*/
#define AFE_INIT_DEFAULT(X) AFE_reg_type X = { \
       .S7_S0   = 0, \
       .LP      = LOW_POWER_DISABLE, \
       .HP      = HP_DISABLE, \
       .C_LP    = LOW_PASS_180, \
       .CLK_OUT = CLK_ENABLE, \
       .SYNC    = MASTER, \
       .TEMP    = TEMP_DISABLE,\
       .INT     = INT_ENABLE \
        }
/*
    .status_bit                = CH_ENABLE,      // Channel enabled
    .Feedback_transconductance = TCONDUCT_1_2T,  // Gain to 1.2T
    .High_pass_filter_setting  = HIGH_PASS_1HZ,  // High pass filter to 1Hz
    .Feedback_cap              = FEEDBACK_50_20, // Feedback cap to 50 fF

*/

#define CCP_INIT_DEFAULT(X) CCP_reg_type X = { \
       .status_bit                = CH_ENABLE, \
       .Feedback_transconductance = TCONDUCT_1_2T, \
       .High_pass_filter_setting  = HIGH_PASS_1HZ, \
       .Feedback_cap              = FEEDBACK_50_20 \
        }

/*
    .UHT          = 0xFF,
    .ULT          = 0xFF,
    .LHT          = 0x00,
    .LLT          = 0x00,
    .WT_Threshold = 0x10,
    .DP0_DP2      = REF_CH_0,
    .CH0_CH2      = SEL_CH_2,
    .ST           = SLEEP_MODE_ONE_CH,

*/
#define WUP_INIT_DEFAULT(X) WUP_reg_type X ={ \
       .UHT = 0xFF, \
       .ULT = 0xFF, \
       .LHT = 0x00, \
       .LLT = 0x00, \
       .WT_Threshold = 0x10, \
       .DP0_DP2 = REF_CH_0, \
       .CH0_CH2 = SEL_CH_2, \
       .ST = SLEEP_MODE_ONE_CH \
         }

#define NORMAL_POWER_INIT_DEFAULT(X) power_settings_type X ={ \
        .S7_S0 = 0x00, \
        .C_LP  = LOW_PASS_180, \
        .CCP_High_pass[0].High_pass_filter_setting = HIGH_PASS_1HZ, \
        .CCP_High_pass[1].High_pass_filter_setting = HIGH_PASS_1HZ, \
        .CCP_High_pass[2].High_pass_filter_setting = HIGH_PASS_1HZ, \
        .CCP_High_pass[3].High_pass_filter_setting = HIGH_PASS_1HZ, \
        .CCP_High_pass[4].High_pass_filter_setting = HIGH_PASS_1HZ, \
        .UHT = 0xFF, \
        .ULT = 0xFF, \
        .LHT = 0x00, \
        .LLT = 0x00, \
        .WT_Threshold = 0x10, \
         }

#define LOW_POWER_INIT_DEFAULT(X) power_settings_type X ={ \
        .S7_S0 = 0x00, \
        .C_LP  = LOW_PASS_30, \
        .CCP_High_pass[0].High_pass_filter_setting = HIGH_PASS_0_17HZ, \
        .CCP_High_pass[1].High_pass_filter_setting = HIGH_PASS_0_17HZ, \
        .CCP_High_pass[2].High_pass_filter_setting = HIGH_PASS_0_17HZ, \
        .CCP_High_pass[3].High_pass_filter_setting = HIGH_PASS_0_17HZ, \
        .CCP_High_pass[4].High_pass_filter_setting = HIGH_PASS_0_17HZ, \
        .UHT = 0xFF, \
        .ULT = 0xFF, \
        .LHT = 0x00, \
        .LLT = 0x00, \
        .WT_Threshold = 0x10, \
         }

/* Private functions  ---------------------------------------------------------*/
/* Initialization and Configuration functions *********************************/
static uint8_t AFE_init(IRPyro_sensor_device_type* this_device);
static uint8_t CCP_init(IRPyro_sensor_device_type* this_device);
static uint8_t WUP_init(IRPyro_sensor_device_type* this_device);
static uint8_t I2C_address_set(IRPyro_sensor_device_type* this_device);

/* FIFO memory functions  *****************************************************/
static uint8_t FIFO_data_active_get(IRPyro_sensor_device_type * this_device);
static uint8_t FIFO_ready_status_get(IRPyro_sensor_device_type* this_device);
static uint8_t FIFO_reset(IRPyro_sensor_device_type* this_device);
static uint8_t IRPyro_test(IRPyro_sensor_device_type* this_device);

/* Support functions  *****************************************************/
static uint8_t AFE_register_empty(AFE_reg_type first);
static uint8_t CCP_register_empty(CCP_reg_type first);
static uint8_t WUP_register_empty(WUP_reg_type first);
static uint8_t channel_value_decode(IRPyro_sensor_device_type* this_device);

static void AFE_to_bytes(AFE_reg_type AFE_register, uint8_t * AFE_byte );
static void CCP_to_bytes(CCP_reg_type CCP_register, uint8_t * CCP_byte );
static void WUP_to_bytes(WUP_reg_type WUP_register, uint8_t * WUP_byte );

static void bytes_to_AFE(uint8_t *AFE_bytes, AFE_reg_type *AFE_register);
static void bytes_to_CCP(uint8_t  CCP_byte,  CCP_reg_type *CCP_register);
static void bytes_to_WUP(uint8_t *WUP_bytes, WUP_reg_type *WUP_register);

/* IO control functions*/
static uint8_t AFE_read(IRPyro_sensor_device_type *this_device, AFE_reg_type *AFE_data);
static uint8_t AFE_write(IRPyro_sensor_device_type *this_device);
static uint8_t CCP_read(IRPyro_sensor_device_type *this_device, CCP_reg_type *CCP_data);
static uint8_t CCP_write(IRPyro_sensor_device_type *this_device);
static uint8_t WUP_read(IRPyro_sensor_device_type *this_device, WUP_reg_type *WUP_data);
static uint8_t WUP_write(IRPyro_sensor_device_type *this_device);

/* Power management functions *************************************************/
static uint8_t IRPyro_power_mode_sleep_set(IRPyro_sensor_device_type* this_device);
static uint8_t IRPyro_power_mode_wakeup_set(IRPyro_sensor_device_type* this_device);

static uint8_t IRPyro_power_settings_init(IRPyro_sensor_device_type* this_device); // this needs to be run before power_mode_set
static uint8_t IRPyro_power_mode_low_set(IRPyro_sensor_device_type* this_device);
static uint8_t IRPyro_power_mode_normal_set(IRPyro_sensor_device_type* this_device);
static struct notice IRPyro_checksum(void);
/**
 * @defgroup IRPyro_sensor_device_type_functions IRPyro sensor device functions
 * @brief Public functions of IRPyro: Open, Read, IOCTL and Close
 * @{
 */
/**
  * @brief  Initializes the IRPyro device.
  * @param  this_device pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
uint8_t IRPyro_open(IRPyro_sensor_device_type* this_device)
{
    uint8_t results_init_sequence[6]= {0};
    uint8_t op_result = 0;

    results_init_sequence[0]= I2C_address_set(this_device);
    results_init_sequence[1]= AFE_init(this_device);
    results_init_sequence[2]= CCP_init(this_device);
    results_init_sequence[3]= WUP_init(this_device);
    results_init_sequence[4]= FIFO_reset(this_device);
    results_init_sequence[5]= IRPyro_power_settings_init(this_device);
    for(uint8_t idx=0; idx<5; ++idx)
    {
        op_result += results_init_sequence[idx];
    }
    op_result = (op_result == 5) ? 1 : 0;  // 1 passed all / 0 fail
    return op_result;
}

/**
  * @brief  Stops streaming, deallocates memory and disable the IRPyro sensor
  * @param  this_device pointer to the IRPyro data structure
  * @retval None
  */
uint8_t IRPyro_close(IRPyro_sensor_device_type* this_device)
{
    CS_pin_set(this_device->CS_Port,this_device->CS_Pin, 0);
    return 1;
}

/**
 * @brief  Reads a frame of data from the IRPyro
 *
 *         Sensor data sampling into the FIFO (SECTION 12.6.1 )
 *         There are two methods of sampling the data available in the FIFO:
 *         Polling mode or Interrupt mode.
 *          - In interrupt mode the host can wait till the interrupt line goes
 *            low in order to read data from the FIFO. The interrupt remains
 *            low till the FIFO is cleared from data.
 *          - In polling mode the host has to read the FIFO status packet to
 *            determine if data is available in order to read it. The number of
 *            data packets available can also be determined from the status
 *            packet allowing multiple reads without having to keep reading the
 *            status packet.
 *         Interrupt mode is a single step operation, polling mode is a two step
 *         operation.
 *
 *     IMPORTANT:
 *       FIFO data collected is stored in the first available position of the
 *       channel_value array.
 *
 *       One channel active:
 *            channel 2 -> channel_value[0]
 *       Four channels active:
 *            channel 1 -> channel_value[0]
 *            channel 2 -> channel_value[1]
 *            channel 3 -> channel_value[2]
 *            channel 4 -> channel_value[3]
 *       For both cases:
 *            frame counter -> channel_value[5]
 *
 * @param  this_device pointer to the IRPyro data structure
 * @retval op_result   1 success / 0 error
 */
uint8_t IRPyro_read(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result = 0;
    // determine type of reading operation
    switch (this_device->read_mode)
    {
    case 0: // read by checking the INT pin status
        this_device->FIFO_register.COUNT = INT_pin_read(this_device->INT_Port, this_device->INT_Pin );
        break;
    case 1: // read by polling
        // query sensor FIFO status
        op_result = FIFO_ready_status_get(this_device);
        break;
    case 2: // read by synchronized sampling: both modes available
        if ((this_device->AFE_register.INT == INT_ENABLE) && (INT_pin_read(this_device->INT_Port, this_device->INT_Pin )))
        {
            this_device->FIFO_register.COUNT =1; // interrupt
        }
        else
        {
            op_result = FIFO_ready_status_get(this_device); // polling
        }
        break;
    default:
        break;
    }
    // read operation
    if (this_device->FIFO_register.COUNT)
    {
        op_result = FIFO_data_active_get(this_device);
        channel_value_decode(this_device);
    }
    return op_result;
}
/**
  * @brief  access to IRPyro registers
  * @param  this_device pointer to the IRPyro data structure
  * @param  cmd pointer to command data structure
  * @param  arg pointer to argument data structure
  * @retval 1 success / 0 fail
  */
uint8_t IRPyro_ioctl(IRPyro_sensor_device_type* this_device, IRPyro_cmd_type *cmd, IRPyro_arg_type arg)
{
    uint8_t op_result = 0;

    switch (cmd->cmd)
    {
    case sampling_rate_set:
        this_device->AFE_register.S7_S0 = arg.arg;
        op_result = AFE_write(this_device);
        break;
    case sampling_rate_get:
        op_result= AFE_read(this_device, &cmd->AFE_register);
        cmd->cmd_return_code = cmd->AFE_register.S7_S0;
        break;
    case high_pass_set:
        this_device->AFE_register.HP = 1;
        this_device->CCP_register[arg.channel].High_pass_filter_setting = arg.arg;
        op_result = (AFE_write(this_device) && CCP_write(this_device));
        break;
    case high_pass_get:
        op_result = (AFE_read(this_device, &cmd->AFE_register) && CCP_read(this_device, cmd->CCP_register));
        cmd->cmd_return_code = this_device->CCP_register[2].High_pass_filter_setting;
        break;
    case low_pass_set:
        this_device->AFE_register.C_LP = arg.arg;
        op_result = AFE_write(this_device);
        break;
    case low_pass_get:
        op_result = AFE_read(this_device, &cmd->AFE_register);
        cmd->cmd_return_code = this_device->AFE_register.C_LP;
        break;
    case gain_set:
        this_device->CCP_register[arg.channel].Feedback_cap = arg.arg;
        op_result= CCP_write(this_device);
        break;
    case gain_get:
        op_result= CCP_read(this_device, cmd->CCP_register);
        cmd->cmd_return_code = this_device->CCP_register[2].Feedback_cap;
        break;
    case power_off:
        op_result= IRPyro_close(this_device);
        break;
    case power_normal:
        op_result= IRPyro_power_mode_normal_set(this_device);
        break;
    case power_low:
        op_result= IRPyro_power_mode_low_set(this_device);
        break;
    case power_sleep:
        op_result= IRPyro_power_mode_sleep_set(this_device);
        break;
    case power_wakeup:
        op_result= IRPyro_power_mode_wakeup_set(this_device);
        break;
    case register_set:
        AFE_write(this_device);
        CCP_write(this_device);
        WUP_write(this_device);
        op_result = 1;
        break;
    case register_get:
        AFE_read(this_device, &cmd->AFE_register);
        CCP_read(this_device, cmd->CCP_register);
        WUP_read(this_device, &cmd->WUP_register);
        op_result = 1;
        break;
    default:
        break;
    }
    return op_result;
}
/** end of IRPyro_API
  * @}
  */
/** @defgroup API_F1_IRPyro Individual sensor initialization and Configuration functions
 *     @brief Default values defined in IRPyro_API_single_device_layer.c can be applied or user assigned
 *  @verbatim
 ===============================================================================
           ##### Initialization and Configuration functions #####
 ===============================================================================
    [..] This section provides a set of functions allowing to initialize the IRPyro sensor,
         AFE, CCP and WUP registers, I2C address.
    @endverbatim
  * @{
  */
/**
 * @defgroup afe_functions AFE register functions
 * @brief Allows to read/write the AFE register performing conversion between
 *           bytes and struct values
 * @{
 */
/**
  * @brief  Initializes the AFE register with this_device values
  * @param  this_device pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t AFE_init(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result = 0;
    AFE_INIT_DEFAULT(afe_defaults);

    if (AFE_register_empty(this_device->AFE_register))
    {
        this_device->AFE_register = afe_defaults;
    }

    op_result = AFE_write(this_device);
    return op_result;
}
/**
  * @brief  reads the AFE register on the IRPyro and loads into this_device
  * @param  this_device pointer to the IRPyro data structure
  * @param  AFE_data    contents of the AFE register on the IRPyro sensor
  * @retval op_result   1 success / 0 error
  */
static uint8_t AFE_read(IRPyro_sensor_device_type *this_device, AFE_reg_type *AFE_data)
{
    uint8_t op_result = 0;
    uint8_t AFE_contents[IRPyro_AFE_SIZE] = {0};

    op_result = i2c_Read(this_device->address, IRPyro_AFE_READ, AFE_contents ,IRPyro_AFE_SIZE);
    bytes_to_AFE(AFE_contents, AFE_data);

    return op_result;
}
/**
  * @brief  writes the AFE register contents of this_device into the IRPyro
  * @param  this_device pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t AFE_write(IRPyro_sensor_device_type *this_device)
{
    uint8_t op_result = 0;
    uint8_t AFE_byte[IRPyro_AFE_SIZE]= {0};

    AFE_to_bytes(this_device->AFE_register, AFE_byte);
    op_result = i2c_Write(this_device->address, IRPyro_AFE_WRITE, AFE_byte ,IRPyro_AFE_SIZE);

    return op_result;
}
/**
  * @brief  encodes the AFE register contents of this_device into the IRPyro
  */
static void AFE_to_bytes(AFE_reg_type AFE_register, uint8_t * AFE_byte )
{
    uint8_t bit_value =0;

    AFE_byte[0] = AFE_register.S7_S0;                 // 0b11111111
    bit_value |= (AFE_register.LP      << 7 )& 0X80 ; // 0b10000000
    bit_value |= (AFE_register.HP      << 6 )& 0X40 ; // 0b01000000
    bit_value |= (AFE_register.C_LP    << 4 )& 0X30 ; // 0b00110000
    bit_value |= (AFE_register.CLK_OUT << 3 )& 0X08 ; // 0b00001000
    bit_value |= (AFE_register.SYNC    << 2 )& 0X04 ; // 0b00000100
    bit_value |= (AFE_register.TEMP    << 1 )& 0X02 ; // 0b00000010
    bit_value |= (AFE_register.INT     << 0 )& 0X01 ; // 0b00000001
    AFE_byte[1] = bit_value;
}
/**
  * @brief  decodes the AFE register of the IRPyro into this_device
  */
static void bytes_to_AFE(uint8_t *AFE_bytes, AFE_reg_type *AFE_register)
{
    AFE_register->S7_S0   = AFE_bytes[0];              // 0b11111111
    AFE_register->LP      = (AFE_bytes[1]& 0X80) >> 7; // 0b10000000
    AFE_register->HP      = (AFE_bytes[1]& 0X40) >> 6; // 0b01000000
    AFE_register->C_LP    = (AFE_bytes[1]& 0X30) >> 4; // 0b00110000
    AFE_register->CLK_OUT = (AFE_bytes[1]& 0X08) >> 3; // 0b00001000
    AFE_register->SYNC    = (AFE_bytes[1]& 0X04) >> 2; // 0b00000100
    AFE_register->TEMP    = (AFE_bytes[1]& 0X02) >> 1; // 0b00000010
    AFE_register->INT     = (AFE_bytes[1]& 0X01) >> 0; // 0b00000001
}
/**@} end of AFE functions
*/
/**
 * @defgroup ccp_functions CCP register functions
 * @brief All enabled channels in the device can be manipulated
 * @{
 */
/**
  * @brief  Initializes the CCP register with this_device values
  * @param  this_device  pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t CCP_init(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result = 0, initialization_failed =0;
    uint8_t selected_channel = 0, ch_to_activate =0;

    CCP_INIT_DEFAULT(ccp_defaults);
    CCP_reg_type CCP_data;
    // active channels value is checked for 1 or 4 channels, if empty defaults to 1
    switch (this_device->number_of_active_channels)
    {
    case 1:
        ch_to_activate   = 1;
        selected_channel = 2;
        break;
    case 4:
        ch_to_activate   = 4;
        selected_channel = 1;
        break;
    default:
        // other than 1 or 4 the initialization fails.
        initialization_failed =1;
        break;
    }
    if (!initialization_failed)
    {
        this_device->number_of_active_channels = ch_to_activate;
        // only active channels are initialized with default data if empty
        while(ch_to_activate)
        {
            CCP_data = this_device->CCP_register[selected_channel];
            if (CCP_register_empty(CCP_data))
            {
                this_device->CCP_register[selected_channel] = ccp_defaults;
            }
            ++selected_channel;
            ch_to_activate--;
        }
        op_result = CCP_write(this_device);
    }
    return op_result;
}
/**
  * @brief  reads the CCP registers on the IRPyro and loads into this_device
  * @param  this_device pointer to the IRPyro data structure
  * @param  CCP_data    contents of the CCP register on the IRPyro sensor
  * @retval op_result   1 success / 0 error
  */
static uint8_t CCP_read(IRPyro_sensor_device_type *this_device, CCP_reg_type *CCP_data)
{
    uint8_t op_result = 0;
    uint8_t CCP_contents[IRPyro_CCP_SIZE] = {0};

    op_result = i2c_Read(this_device->address, IRPyro_CCP_READ, CCP_contents ,IRPyro_CCP_SIZE);
    for(uint8_t idx=0; idx < IRPyro_CCP_SIZE; ++idx)
    {
        bytes_to_CCP(CCP_contents[idx], &CCP_data[idx]);
    }
    return op_result;
}
/**
  * @brief  writes the CCP registers contents of this_device into the IRPyro
  * @param  this_device pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t CCP_write(IRPyro_sensor_device_type *this_device)
{
    uint8_t op_result = 0;
    uint8_t CCP_byte[IRPyro_CCP_SIZE] = {0};

    for(uint8_t idx=0; idx < IRPyro_CCP_SIZE; ++idx)
    {
        CCP_to_bytes(this_device->CCP_register[idx], &CCP_byte[idx]);
    }
    op_result = i2c_Write(this_device->address, IRPyro_CCP_WRITE, CCP_byte ,IRPyro_CCP_SIZE);

    return op_result;
}
/**
  * @brief  encodes the CCP register contents of this_device into IRPyro
  */
static void CCP_to_bytes(CCP_reg_type CCP_register, uint8_t *CCP_byte )
{
    uint8_t bit_value = 0;

    bit_value |= (CCP_register.Feedback_transconductance << 6)& 0XC0 ; // 0b11000000
    bit_value |= (CCP_register.High_pass_filter_setting  << 4)& 0X30 ; // 0b00110000
    bit_value |= (CCP_register.Feedback_cap              << 1)& 0X0E ; // 0b00001110
    bit_value |= (CCP_register.status_bit                << 0)& 0X01 ; // 0b00000001
    *CCP_byte = bit_value;
}
/**
  * @brief  decodes the CCP register of the IRPyro into this_device
  */
static void bytes_to_CCP(uint8_t CCP_byte, CCP_reg_type *CCP_register)
{
    CCP_register->Feedback_transconductance = (CCP_byte & 0XC0 )>> 6 ; // 0b11000000
    CCP_register->High_pass_filter_setting  = (CCP_byte & 0X30 )>> 4 ; // 0b00110000
    CCP_register->Feedback_cap              = (CCP_byte & 0X0E )>> 1 ; // 0b00001110
    CCP_register->status_bit                = (CCP_byte & 0X01 )>> 0 ; // 0b00000001
}
/**@} end of CCP functions
*/
/**
 * @defgroup WUP_functions WUP register functions
 * @brief Modifies the WUP register
 * @{
 */
/**
  * @brief  Initializes the WUP register with this_device values
  * @param  this_device pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t WUP_init(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result  = 0;
    WUP_INIT_DEFAULT(wup_defaults);

    if (WUP_register_empty(this_device->WUP_register))
    {
        this_device->WUP_register = wup_defaults;
    }

    op_result = WUP_write(this_device);
    return op_result;
}
/**
  * @brief  reads the WUP register on the IRPyro and loads into this_device
  * @param  this_device pointer to the IRPyro data structure
  * @param  WUP_data    pointer to the WUP data structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t WUP_read(IRPyro_sensor_device_type *this_device, WUP_reg_type *WUP_data)
{
    uint8_t op_result = 0;
    uint8_t WUP_contents[IRPyro_WUP_SIZE] = {0};

    op_result = i2c_Read(this_device->address, IRPyro_WUP_READ, WUP_contents ,IRPyro_WUP_SIZE);
    bytes_to_WUP(WUP_contents, WUP_data );

    return op_result;
}
/**
  * @brief  writes the WUP register contents of this_device into the IRPyro
  * @param  this_device pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t WUP_write(IRPyro_sensor_device_type *this_device)
{
    uint8_t op_result = 0;
    uint8_t WUP_byte[IRPyro_WUP_SIZE] = {0};

    WUP_to_bytes(this_device->WUP_register, WUP_byte);
    op_result = i2c_Write(this_device->address, IRPyro_WUP_WRITE, WUP_byte ,IRPyro_WUP_SIZE);

    return op_result;
}
/**
  * @brief  encodes the WUP register contents of this_device into the IRPyro
  */
static void WUP_to_bytes(WUP_reg_type WUP_register, uint8_t * WUP_byte)
{
    uint8_t bit_value = 0;

    WUP_byte[0] = WUP_register.UHT;
    WUP_byte[1] = WUP_register.ULT;
    WUP_byte[2] = WUP_register.LHT;
    WUP_byte[3] = WUP_register.LLT;
    WUP_byte[4] = WUP_register.WT_Threshold;

    bit_value |= (WUP_register.ST      << 6)& 0X40 ; // 0b01000000
    bit_value |= (WUP_register.DP0_DP2 << 3)& 0X38 ; // 0b00111000
    bit_value |= (WUP_register.CH0_CH2 << 0)& 0X07 ; // 0b00000111
    WUP_byte[5] = bit_value;
}
/**
  * @brief  decodes the WUP register of the IRPyro into this_device
  */
static void bytes_to_WUP(uint8_t *WUP_bytes, WUP_reg_type *WUP_register)
{
    WUP_register->UHT          = WUP_bytes[0];
    WUP_register->ULT          = WUP_bytes[1];
    WUP_register->LHT          = WUP_bytes[2];
    WUP_register->LLT          = WUP_bytes[3];
    WUP_register->WT_Threshold = WUP_bytes[4];
    WUP_register->ST           =(WUP_bytes[5] & 0X40) >> 6; // 0b01000000
    WUP_register->DP0_DP2      =(WUP_bytes[5] & 0X38) >> 3; // 0b00111000
    WUP_register->CH0_CH2      =(WUP_bytes[5] & 0X07) >> 0; // 0b00000111
}
/**@} end of WUP functions
*/
/**
  * @brief  Sends test code to the IRPyro sensor to confirm is there.
  * @param  this_device pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t IRPyro_test(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result = 0;
    uint8_t i2c_result = 0;

    i2c_result = i2c_Read(this_device->address, IRPyro_TEST, &op_result ,1);
    if ((i2c_result == 1) && (op_result == IRPyro_TEST_OK))
    {
        op_result =1;
    }
    else
        op_result = i2c_result;

    return op_result;
}
/**
  * @brief  Sends new I2C address to the IRPyro sensor
  *         Default i2c address is 0x65, if multiple devices are being used
  *         care needs to be taken when writing new i2c addresses by switching off
  *         other devices with the same address.
  * @param  this_device pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t I2C_address_set(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result = 0;
    uint8_t i2c_result = 0;
    uint8_t attempt = 3;
    do
    {
        if(this_device->CS_Port>0 && this_device->CS_Pin>0)
        {
            power_set(this_device->CS_Port, this_device->CS_Pin);
        }
        else
        {
            Micro_controller_delay_ms(1);
        }
        i2c_result= i2c_Read(IRPyro_DEFAULT_I2C, IRPyro_RESET_FULL, &op_result ,1);
        if ((i2c_result == 1) && (op_result == IRPyro_RESET_FULL_OK))
        {
            op_result =1;
            break;
        }
        else
            op_result = i2c_result;

    }
    while (--attempt);

    Micro_controller_delay_us(100);

    if (op_result == 1)
    {
        uint8_t IRPyro_Address = this_device->address <<1;
        i2c_result =  i2c_Write(IRPyro_DEFAULT_I2C, IRPyro_ADDR_WRITE, &IRPyro_Address ,1);
        if ((i2c_result == 1) && (op_result == IRPyro_RESET_FULL_OK))
        {
            op_result =1;
        }
        else
            op_result = i2c_result;
    }
    return op_result;
}
/**
  * @}
  */
/** @defgroup API_F8_IRPyro FIFO memory functions
 *  @brief   FIFO memory functions
 *
@verbatim
 ===============================================================================
           #####  FIFO memory functions #####
 ===============================================================================
    [..] This section provides a set of functions allowing to get captured data
         from the sensor RAM register
@endverbatim
  * @{
  */
/**
 * @brief  Read data packet (up to 17 bytes) for only active channel
 * @param  this_device  pointer to the IRPyro data structure
 * @retval op_result   1 success / 0 error
  */
static uint8_t FIFO_data_active_get(IRPyro_sensor_device_type * this_device)
{
    uint8_t bytes_to_read = IRPyro_BYTES_PER_CHANNEL * this_device->number_of_active_channels + IRPyro_BYTES_FRAME_COUNT;
    uint8_t op_result = 0;
    uint8_t i2c_result = 0;

    i2c_result = i2c_Read(this_device->address, IRPyro_FIFO_READ_ACTIVE, this_device->raw_data, bytes_to_read);
    if (i2c_result == 1)
    {
        i2c_result = i2c_Read(this_device->address, IRPyro_FIFO_CLEAR, &op_result, 1);
        if ((i2c_result == 1) && (op_result == IRPyro_FIFO_CLEAR_OK))
        {
            op_result = 1;
        }
        else
            op_result = i2c_result;
    }
    else
        op_result = i2c_result;

    return op_result;
}
/**
  * @brief  Ask the IRPyro if a frame of data is available
  *         used when a polling reading operation has started
  * @param  this_device         pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t FIFO_ready_status_get(IRPyro_sensor_device_type* this_device)
{
    uint8_t IRPyro_Sensor_FIFO_status = 0;
    uint8_t op_result = 0;

    op_result = i2c_Read(this_device->address, IRPyro_FIFO_STATUS, &IRPyro_Sensor_FIFO_status, IRPyro_FIFO_STATUS_SIZE);
    this_device->FIFO_register.WAKE        = (IRPyro_Sensor_FIFO_status & IRPyro_FIFO_WAKE_MASK );
    this_device->FIFO_register.STATUS_CODE = (IRPyro_Sensor_FIFO_status & IRPyro_FIFO_ERROR_MASK ) >> IRPyro_SHIFT_FOUR;
    this_device->FIFO_register.COUNT       = (IRPyro_Sensor_FIFO_status & IRPyro_FIFO_COUNT_MASK ) >> IRPyro_SHIFT_ONE;
    this_device->FIFO_register.DATA_READY  = (IRPyro_Sensor_FIFO_status & IRPyro_FIFO_INT_MASK );

    return op_result;
}
/**
  * @brief  Clear the entire FIFO (Rd/Wr pointer reset with the Empty)
  * @param  this_device pointer to the IRPyro data structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t FIFO_reset(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result = 0;
    uint8_t i2c_result = 0;

    i2c_result = i2c_Read(this_device->address, IRPyro_FIFO_RESET, &op_result ,1);
    if ((i2c_result == 1) && (op_result == IRPyro_FIFO_RESET_OK))
    {
        op_result =1;
    }
    else
        op_result = i2c_result;
    return op_result;
}
/** end of API_F8_IRPyro FIFO memory functions
  * @}
  */

/** @defgroup  API_F2_IRPyro Power management functions
 *  @brief   Power management functions
 *
@verbatim
 ===============================================================================
           ##### Power management functions #####
 ===============================================================================
    [..] This section provides a set of functions allowing to control the power
         mode of the IRPyro sensor.

@endverbatim
  * @{
  */
/**
  * @brief  The device will automatically enter either low power or normal power sleep mode
  *         depending in the power state the device is in when it is put to sleep.
  *         The WUP register contains the settings for either sleep mode.
  * @param  this_device pointer to the IRPyro data structure
  * @retval 0 fail /1 success
  */
static uint8_t IRPyro_power_mode_sleep_set(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result = 0;
    uint8_t i2c_result = 0;

    i2c_result = i2c_Read(this_device->address, IRPyro_GO_TO_SLEEP, &op_result ,1);
    if ((i2c_result == 1) && (op_result == IRPyro_GO_TO_SLEEP_OK))
    {
        op_result =1;
    }
    else
        op_result = i2c_result;

    return op_result;
}

/**
  * @brief  The device will automatically enter either low power or normal power sleep mode
  *         depending in the power state the device is in when it is put to sleep.
  *         The WUP register contains the settings for either sleep mode.
  * @param  this_device pointer to the IRPyro data structure
  * @retval 0 fail /1 success
  */
static uint8_t IRPyro_power_mode_wakeup_set(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result = 0;
    uint8_t i2c_result = 0;
    i2c_result = i2c_Write(this_device->address, IRPyro_WAKE_UP, 0,0);
    if (i2c_result == 1)
    {
        op_result = IRPyro_test(this_device);
    }
    else
        op_result = i2c_result;

    return op_result;                         //(op_result == IRPyro_WAKE_UP_OK ? 1:0);
}
/**
  * @brief initializes with defaults the power settings of an IRPyro device
  */
static uint8_t IRPyro_power_settings_init(IRPyro_sensor_device_type* this_device)
{
    NORMAL_POWER_INIT_DEFAULT(normal_power_init);
    LOW_POWER_INIT_DEFAULT(low_power_init);

    this_device->normal_power = normal_power_init;
    this_device->low_power    = low_power_init;

    return 1;
}
/**
  * @brief activates the low power mode settings stored on the IRPyro structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t IRPyro_power_mode_low_set(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result = 0;
    this_device->AFE_register.LP = 1;
    this_device->AFE_register.S7_S0 = this_device->low_power.S7_S0;
    this_device->AFE_register.C_LP  = this_device->low_power.C_LP;

    this_device->CCP_register[0].High_pass_filter_setting  = this_device->low_power.CCP_High_pass[0].High_pass_filter_setting;
    this_device->CCP_register[1].High_pass_filter_setting  = this_device->low_power.CCP_High_pass[1].High_pass_filter_setting;
    this_device->CCP_register[2].High_pass_filter_setting  = this_device->low_power.CCP_High_pass[2].High_pass_filter_setting;
    this_device->CCP_register[3].High_pass_filter_setting  = this_device->low_power.CCP_High_pass[3].High_pass_filter_setting;
    this_device->CCP_register[4].High_pass_filter_setting  = this_device->low_power.CCP_High_pass[4].High_pass_filter_setting;

    op_result = AFE_write(this_device) && CCP_write(this_device) ?1:0;

    return op_result;
}
/**
  * @brief activates the normal power mode settings stored on the IRPyro structure
  * @retval op_result   1 success / 0 error
  */
static uint8_t IRPyro_power_mode_normal_set(IRPyro_sensor_device_type* this_device)
{
    uint8_t op_result = 0;
    this_device->AFE_register.LP = 0;
    this_device->AFE_register.S7_S0 = this_device->normal_power.S7_S0;
    this_device->AFE_register.C_LP  = this_device->normal_power.C_LP;

    this_device->CCP_register[0].High_pass_filter_setting  = this_device->normal_power.CCP_High_pass[0].High_pass_filter_setting;
    this_device->CCP_register[1].High_pass_filter_setting  = this_device->normal_power.CCP_High_pass[1].High_pass_filter_setting;
    this_device->CCP_register[2].High_pass_filter_setting  = this_device->normal_power.CCP_High_pass[2].High_pass_filter_setting;
    this_device->CCP_register[3].High_pass_filter_setting  = this_device->normal_power.CCP_High_pass[3].High_pass_filter_setting;
    this_device->CCP_register[4].High_pass_filter_setting  = this_device->normal_power.CCP_High_pass[4].High_pass_filter_setting;

    op_result = AFE_write(this_device) && CCP_write(this_device) ?1:0;

    return op_result;
}
/** end of API_F2_IRPyro Power management functions
  * @}
  */

/** @defgroup API_F10_IRPyro Support functions
 *  @brief   Data structures manipulation functions
 *
@verbatim
 ===============================================================================
           #####  Support functions  #####
 ===============================================================================
    [..] This section provides a set of auxiliary functions

@endverbatim
  * @{
  */
/**
  * @brief  Decodes the channel values on a given frame
  * @param  this_device           pointer to IRPyro device
  * @retval ch_value              pointer to IRPyro decoded channel data
  * @verbatim
  * Up to 17 Bytes containing the top frame available in the FIFO which has only
  * the active channel data as well as a frame counter (the example below shows
  * channel 0 and 4 active):
  *
  * FIFO_DPA    Data
  * Byte 0  channel 0 bits 23-16
  * Byte 1  channel 0 bits 15-8
  * Byte 2  channel 0 bits 7-0
  * Byte 3  channel 4 bits 23-16
  * Byte 4  channel 4 bits 15-8
  * Byte 5  channel 4 bits 7-0
  * Byte 6  frame count MSB
  * Byte 7  frame count LSB
  *
  * Channel Data : 24 bits unsigned value
  * Frame count : 16 bits unsigned value
  * In this mode, the number of Bytes sent will depend on number of active channels e.g.
  * 2 channels (ch0 and ch4) active = 2 * (3 Bytes) + (2 Bytes frame counter) = 8 Bytes
  *
  * @endverbatim
  */
static uint8_t channel_value_decode(IRPyro_sensor_device_type* this_device)
{
    uint8_t bytes_to_read = 0;
    uint8_t ch_saturated  = 0;

    // for number of active channels
    uint8_t channels_to_decode = 5;
    uint8_t channel_in_process = 0;
    uint8_t update_number_of_active_channels = 0;
    uint8_t channel_array[5] = {0};
    uint8_t channel_array_index = 0;

    do
    {
        if(this_device->CCP_register[channel_in_process].status_bit)
        {
            channel_array[channel_array_index] = channel_in_process;
            ++ update_number_of_active_channels;
            ++ channel_array_index;
        }
        -- channels_to_decode;
        ++ channel_in_process;
    }
    while (channels_to_decode);

    this_device->number_of_active_channels = update_number_of_active_channels;
    bytes_to_read = IRPyro_BYTES_PER_CHANNEL * this_device->number_of_active_channels + IRPyro_BYTES_FRAME_COUNT; // IRPyro_DATA_SIZE;
    for(uint8_t current_channel=0; current_channel<this_device->number_of_active_channels; ++current_channel)
    {
        // decode the value of the channel
        uint8_t ch_data_pos;
        uint8_t final_data_location;
        ch_data_pos = (current_channel*IRPyro_BYTES_PER_CHANNEL);
        final_data_location = channel_array[current_channel];

        this_device->channel_value[final_data_location] = (this_device->raw_data[ch_data_pos]<<16) + (this_device->raw_data[ch_data_pos+1]<<8) + this_device->raw_data[ch_data_pos+2];
        ch_saturated += (this_device->channel_value[final_data_location] & IRPyro_CHANNEL_SAT) >> (23 - current_channel);
    }

    this_device->channel_value[IRPyro_FRAME_COUNT_LOCATION] = (this_device->raw_data[bytes_to_read - IRPyro_FRAME_COUNT_TOP_BYTE] << IRPyro_SHIFT_TO_MSB) + this_device->raw_data[bytes_to_read- IRPyro_FRAME_COUNT_LOW_BYTE];
    this_device->channel_saturation_byte = ch_saturated;
    return 1;
}
/**
  * @brief  Checks if the AFE register is empty
  * @param  first    AFE register
  * @retval result_empty 1 empty / 0 not empty
  */
static uint8_t AFE_register_empty(AFE_reg_type first)
{
    uint8_t i = 0, result_empty = 0;
    uint8_t *AFE_pointer_one = &(first.S7_S0);
    uint8_t AFE_size = sizeof(first);

    while (i<AFE_size)
    {
        result_empty +=AFE_pointer_one[i];
        ++i;
    }
    return (result_empty == 0 ?1:0);
}
/**
  * @brief  Checks if the CCP register is empty
  * @param  first    CCP register
  * @retval result_empty 1 empty / 0 not empty
  */
static uint8_t CCP_register_empty(CCP_reg_type first)
{
    uint8_t i = 0, result_empty = 0;
    uint8_t *CCP_pointer_one = &(first.Feedback_cap);
    uint8_t CCP_size = sizeof(first);

    while (i<CCP_size)
    {
        result_empty +=CCP_pointer_one[i];
        ++i;
    }
    return (result_empty == 0 ?1:0);
}
/**
  * @brief  Checks if the AFE register is empty
  * @param  first    First AFE
  * @retval result_empty 1 empty / 0 not empty
  */
static uint8_t WUP_register_empty(WUP_reg_type first)
{
    uint8_t i = 0, result_empty = 0;
    uint8_t *WUP_pointer_one = &(first.UHT);
    uint8_t WUP_size = sizeof(first);

    while (i<WUP_size)
    {
        result_empty +=WUP_pointer_one[i];
        ++i;
    }
    return (result_empty == 0 ?1:0);
}
/**
  * @brief  Checksum string for the IRPyro API
  */
struct notice
{
    char IRPyro_API_notice[35];
};
static struct notice IRPyro_checksum(void)
{
    struct notice legend = {0x43 ,0x6f ,0x70 ,0x79 ,0x72 ,0x69 ,0x67 ,0x68 ,0x74 ,0x20 ,0x28 ,0x63 ,0x29 ,0x20 ,0x32 ,0x30 ,0x31 ,0x38 ,0x20 ,0x50 ,0x79 ,0x72 ,0x65 ,0x6f ,0x73 ,0x2c ,0x20 ,0x4c ,0x74 ,0x64 ,0x2e};
    return legend;
}
/** end of Support functions
  * @}
  */
/** end of IRPyro Single Device Layer
  * @}
  */
/** end of IRPyro_API
  * @}
  */

/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
