/**
******************************************************************************
* @file    main.c
* @author  _KEMET, Ltd
* @date    March 2018
* @version Release 1.0.6
* @copyright (c) 2018 _KEMET, Ltd
* @brief   Examples of single sensor and multiple sensor streaming. Also IOCTL
*          of a single sensor.
*
* @verbatim
 ===============================================================================
                      ##### How to use this example #####
 ===============================================================================
  [..]
  Requirements
   (#) The IRPyro API (IRPyroAPI.c)
   (#) IRPyro breakout board (IRPYxxxxx-B1), IRPyro Base Board (dIRBB01) or
       an IRPyro demo kit for motion, gas or flame. (See _KEMET website)
   (#) Keil compiler version 5 onwards (https://www.keil.com/download/product/)
       to build this example.
   (#) C99 option enabled on the compiler to support initialization macros.
   (#) The STM32 Standard Peripheral Library, see:
      (http://www.st.com/content/st_com/en/products/embedded-software/mcus-embedded-software/stm32-embedded-software/stm32-standard-peripheral-libraries.html)
  [..]
   (#) See Micro_controller_template.c for details on how to configure SDA,SCL
       INT and CS lines values.
   (#) Create a variable of IRPyro_sensor_device_type
   (#) Assign the values of the port and pin for the CS and INT lines
   (#) Fill the AFE, CCP, and WUP registers with the desired values
   (#) Compile, load and run.
   (#) Display collected data on RealTerm or equivalent
 @endverbatim
  ******************************************************************************
   @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>
   @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.

    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim

  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "main.h"

static void read_a_sample(IRPyro_sensor_device_type* this_device);
static void read_sample_single_device(void);
static void read_sample_multiple_devices(void);
static void IOCTL_example(void);
static void wake_up_event_detection(void);
static void demo_board_sensor_control(int value);
static void usart_callback_setup(void);
static void IRPyro_sensor_population(IRPyro_devices this_list, uint8_t *list_size);

/** @defgroup IRPyroAPI_Examples Code examples
  * @brief How to stream IRPyro data and change parameters at high level
  * @{
  */
/**
  * @brief Call point to the example functions
  * @attention  Uncomment the function to run
  */
int main(void)
{
    read_sample_single_device();    // use with single sensor board
//    read_sample_multiple_devices(); // use with multiple sensors
//    wake_up_event_detection();      // use with single sensor board
//    IOCTL_example();                // use with single sensor board
}
/**
  * @brief Example to initialize one IRPyro and stream data
  *
  *        Creates an IRPyro sensor with all four channels active and the minimal
  *        required configuration: location of the CS and INT port/pin and a I2C
  *        address. See Read_Me.txt
  * @attention Select display format to the UART: UART_stream_char_send() or
  *            UART_stream_binary_send() for plain text or binary format package.
  * @note  Modify accordingly depending on architecture
  */
static void read_sample_single_device(void)
{
    /*------------------------- IRPyro sensor definition ---------------------*/
    /*
     * NOTE: .number_of_active_channels can be set to 1 or 4.
     *       - when equal to 1 data from Chanel 2 is collected
     *       - when equal to 4 data from channels 1-4 is collected.
     * IMPORTANT:
     *       FIFO data collected is stored in the first available position of the
     *       channel_value array.
     *
     *       One channel active:
     *            channel 2 -> channel_value[0]
     *       Four channels active:
     *            channel 1 -> channel_value[0]
     *            channel 2 -> channel_value[1]
     *            channel 3 -> channel_value[2]
     *            channel 4 -> channel_value[3]
     *       For both cases:
     *            frame counter -> channel_value[5]
     */
#define NUMBER_OF_SAMPLES_TO_COUNT 6000000
    /*-------------------------- memory allocation----------------------------*/
    IRPyro_sensor_device_type IRPyro_Sensor =
    {
        .CS_Port  = 2,
        .CS_Pin   = 1,
        .INT_Port = 1,
        .INT_Pin  = 4,
        .address  = 0x75,
        .number_of_active_channels = 1,
        .read_mode=0,
    };

    /*----------------------- --- CONFIGURATION ------------------------------*/

    AFE_reg_type IRPyro_Sensor_AFE =
    {
        .S7_S0   = 0,                  // Sets the sampling rate to 1,000 samples per second.
        .LP      = LOW_POWER_DISABLE,  // Low power sets the sampling rate to 166 samples per second.
        .HP      = HP_DISABLE,         // High pass filter enabled
        .C_LP    = LOW_PASS_180,       // Low pass filter set at 180Hz
        .CLK_OUT = CLK_ENABLE,         // Clock output enabled to synchronize other IRPyro sensors
        .SYNC    = MASTER,             // IRPyro set as Master issues the synchronization to SLAVE
        .TEMP    = TEMP_DISABLE,       //
        .INT     = INT_ENABLE          // IRPyro indicates data ready enable
    };

    CCP_reg_type IRPyro_Sensor_Ch_Cfg =
    {
        .Feedback_cap              = FEEDBACK_200_80, // Feedback capacitance
        .High_pass_filter_setting  = HIGH_PASS_1HZ,   // High pass filter set at 1Hz
        .Feedback_transconductance = TCONDUCT_1_2T,   // Time constant selection
        .status_bit                = CH_ENABLE        // Channel ready to collect data
    };

    WUP_reg_type IRPyro_Sensor_WUP_Cfg =
    {
        .UHT          = MAX_WUP_THRESHOLD,            // Upper High Threshold = 255 events
        .ULT          = SAFE_WUP_THRESHOLD,           // Upper Low  Threshold = 180 events
        .LHT          = MIN_WUP_THRESHOLD,            // Low   High Threshold =   0 events
        .LLT          = MIN_WUP_THRESHOLD,            // Low   Low  Threshold =   0 events
        .WT_Threshold = DEFAULT_NO_EVENTS,            // Wake up event Threshold = 10 events
        .DP0_DP2      = NOT_IN_USE_VALUE,             // Active pixel channel
        .CH0_CH2      = SEL_CH_2,                     // Reference channel
        .ST           = SLEEP_MODE_ONE_CH             // Sleep mode type
    };

    IRPyro_Sensor.AFE_register    = IRPyro_Sensor_AFE;
    IRPyro_Sensor.CCP_register[2] = IRPyro_Sensor_Ch_Cfg;
    IRPyro_Sensor.WUP_register    = IRPyro_Sensor_WUP_Cfg;

    /*-------------------------- board setup ---------------------------------*/
    Micro_controller_template_init();
    usart_callback_setup();
    // disable all CS lines
    demo_board_sensor_control(0);
    /*----------------- enable the sensor for streaming-----------------------*/
    IRPyro_open(&IRPyro_Sensor);

    for(int samples_counter = 0; samples_counter<NUMBER_OF_SAMPLES_TO_COUNT; ++samples_counter)
    {
        /*------------------------- application variables ------------------------*/

        /*-----------read the collected data and store in memory -----------------*/
        IRPyro_read(&IRPyro_Sensor);
        /*--------------- send collected data to a host --------------------------*/
        UART_stream_char_send(IRPyro_Sensor.channel_value, IRPyro_Sensor.channel_saturation_byte);
        //UART_stream_binary_send(IRPyro_Sensor.raw_data);
    }
    /*---------------- do what is needed with the value ----------------------*/


    /*--------------------- close the device ---------------------------------*/
    IRPyro_close(&IRPyro_Sensor);
}
/**
 * @brief  IRPyro data collection for a number samples
 * @param  this_device          IRPyro configuration data
 * @retval number_of_frames_lost
 * @attention Select display format to the UART: UART_stream_char_send() or
 *            UART_stream_binary_send() for plain text or binary format package.
 */
static void read_a_sample(IRPyro_sensor_device_type* this_device)
{
#define NUMBER_OF_FRAMES_TO_COUNT    3      // change to the desired number
#define IRPyro_FRAME_COUNT_LOCATION  5      // position on the user data buffer
    uint16_t sample_counter        = 0;
    int read_delay = 1;

    /* A delay is created based on the power mode and the sample rate configured
       to avoid reading the device too fast.
    */
    if(this_device->AFE_register.C_LP)
    {
        read_delay = (this_device->AFE_register.S7_S0 +1)*6;  // on low power approximate factor
    }
    else
    {
        read_delay = this_device->AFE_register.S7_S0 +1;
    }
    IRPyro_read(this_device);
    while (sample_counter < NUMBER_OF_FRAMES_TO_COUNT)
    {
        Micro_controller_delay_ms(read_delay);
        if (IRPyro_read(this_device))
        {
            ++sample_counter;
            UART_stream_char_send(this_device->channel_value, this_device->channel_saturation_byte);
            // UART_stream_binary_send(this_device->raw_data);
        }
    }
}
/**
  * @brief Example to initialize multiple IRPyros and stream data
  *
  *        Creates a list of IRPyro sensors with all four channels active and the minimal
  *        required configuration: location of the CS and INT port/pin and a I2C
  *        address. See Read_Me.txt
  * @attention Select display format to the UART: UART_stream_char_send() or
  *            UART_stream_binary_send() for plain text or binary format package.
  * @note  Modify accordingly depending on architecture
  */
static void read_sample_multiple_devices(void)
{
    /*------------------------- IRPyro sensor definition ---------------------*/
    /*
     * NOTE: .number_of_active_channels can be set to 1 or 4.
     *       - when equal to 1 data from channel 2 is collected
     *       - when equal to 4 data from channels 1-4 is collected.
     * IMPORTANT:
     *   FIFO data collected is stored in each individual sensor data structure
     *   using the same criteria as with a single sensor. See stream_data_IRPyro()
     */
#define IRPyro_MAXINT 2147483647
    /*------------------- Memory allocation, initialization ------------------*/

    IRPyro_devices IRPyro_device = {0};

    /*----------------------- --- CONFIGURATION -----  -----------------------*/

    AFE_reg_type IRPyro_AFE_Master =
    {
        .S7_S0   = 0,                  // Sets the sampling rate to 1,000 samples per second.
        .LP      = LOW_POWER_DISABLE,  // Low power sets the sampling rate to 166 samples per second.
        .HP      = HP_DISABLE,         // High pass filter enabled
        .C_LP    = LOW_PASS_180,       // Low pass filter set at 180Hz
        .CLK_OUT = CLK_ENABLE,         // Clock output enabled to synchronize other IRPyro sensors
        .SYNC    = MASTER,             // IRPyro set as Master issues the synchronization to SLAVE
        .TEMP    = TEMP_DISABLE,       //
        .INT     = INT_ENABLE          // IRPyro indicates data ready enable
    };
    AFE_reg_type IRPyro_AFE_Slave =
    {
        .S7_S0   = 0,                  // Sets the sampling rate to 1,000 samples per second.
        .LP      = LOW_POWER_DISABLE,  // Low power sets the sampling rate to 166 samples per second.
        .HP      = HP_DISABLE,         // High pass filter enabled
        .C_LP    = LOW_PASS_180,       // Low pass filter set at 180Hz
        .CLK_OUT = CLK_DISABLE,        // Clock output enabled to synchronize other IRPyro sensors
        .SYNC    = SLAVE,              // IRPyro set as Slave receives synchronization from Master
        .TEMP    = TEMP_DISABLE,       //
        .INT     = INT_ENABLE          // IRPyro indicates data ready enable
    };

    CCP_reg_type IRPyro_Ch_Cfg =
    {
        .Feedback_cap              = FEEDBACK_200_80, // Feedback capacitance
        .High_pass_filter_setting  = HIGH_PASS_1HZ,   // High pass filter set at 1Hz
        .Feedback_transconductance = TCONDUCT_1_2T,   // Time constant selection
        .status_bit                = CH_ENABLE        // Channel ready to collect data
    };

    WUP_reg_type IRPyro_WUP_Cfg =
    {
        .UHT          = MAX_WUP_THRESHOLD,            // Upper High Threshold = 255 events
        .ULT          = SAFE_WUP_THRESHOLD,           // Upper Low  Threshold = 180 events
        .LHT          = MIN_WUP_THRESHOLD,            // Low   High Threshold =   0 events
        .LLT          = MIN_WUP_THRESHOLD,            // Low   Low  Threshold =   0 events
        .WT_Threshold = DEFAULT_NO_EVENTS,            // Wake up time Threshold = 10 events
        .DP0_DP2      = NOT_IN_USE_VALUE,             // Active pixel channel
        .CH0_CH2      = SEL_CH_2,                     // Reference channel
        .ST           = SLEEP_MODE_ONE_CH             // Sleep mode type
    };
    uint8_t num_of_IRPyro_on_demo_board = 4;   // number of devices on a demo board
    /*-----------------    ADDRESSES   ------------*/
    /* Assuming four sensors on an IRPyro Break out board or demo board*/
    IRPyro_device[0].INT_Port = 1;
    IRPyro_device[0].CS_Port  = 2;
    IRPyro_device[0].CS_Pin   = 1;
    IRPyro_device[0].INT_Pin  = 4;
    IRPyro_device[0].address  = 0x70;

    IRPyro_device[1].CS_Port  = 2;
    IRPyro_device[1].INT_Port = 1;
    IRPyro_device[1].CS_Pin   = 5;
    IRPyro_device[1].INT_Pin  = 5;
    IRPyro_device[1].address  = 0x69;

    IRPyro_device[2].CS_Port  = 2;
    IRPyro_device[2].INT_Port = 1;
    IRPyro_device[2].CS_Pin   = 4;
    IRPyro_device[2].INT_Pin  = 6;
    IRPyro_device[2].address  = 0x68;

    IRPyro_device[3].CS_Port  = 2;
    IRPyro_device[3].INT_Port = 1;
    IRPyro_device[3].CS_Pin   = 3;
    IRPyro_device[3].INT_Pin  = 7;
    IRPyro_device[3].address  = 0x67;

    /*-----------------    CONFIGURATION   ------------*/
    IRPyro_device[0].number_of_active_channels =1;
    IRPyro_device[0].AFE_register    = IRPyro_AFE_Master; // MASTER, INT, CLK ENABLED
    IRPyro_device[0].CCP_register[2] = IRPyro_Ch_Cfg;
    IRPyro_device[0].WUP_register    = IRPyro_WUP_Cfg;

    IRPyro_device[1].number_of_active_channels =1;
    IRPyro_device[1].AFE_register    = IRPyro_AFE_Slave; // SLAVE, INT, CLK DISABLED
    IRPyro_device[1].CCP_register[2] = IRPyro_Ch_Cfg;
    IRPyro_device[1].WUP_register    = IRPyro_WUP_Cfg;

    IRPyro_device[2].number_of_active_channels =1;
    IRPyro_device[2].AFE_register    = IRPyro_AFE_Slave; // SLAVE, INT, CLK DISABLED
    IRPyro_device[2].CCP_register[2] = IRPyro_Ch_Cfg;
    IRPyro_device[2].WUP_register    = IRPyro_WUP_Cfg;

    IRPyro_device[3].number_of_active_channels =1;
    IRPyro_device[3].AFE_register    = IRPyro_AFE_Slave; // SLAVE, INT, CLK DISABLED
    IRPyro_device[3].CCP_register[2] = IRPyro_Ch_Cfg;
    IRPyro_device[3].WUP_register    = IRPyro_WUP_Cfg;

    /*-------------------------- board setup ---------------------------------*/
    Micro_controller_template_init();
    usart_callback_setup();
    IRPyro_sensor_population(IRPyro_device, &num_of_IRPyro_on_demo_board);
    // disable all CS lines
    demo_board_sensor_control(0);
    /*-------- test the open_list, read_list and close_list functions --------*/
    /*-------------------------- open_list   ---------------------------------*/
    IRPyro_open_devices(IRPyro_device, num_of_IRPyro_on_demo_board);
    /*-------------------------- read_list   ---------------------------------*/
    for (int sweep=0; sweep< IRPyro_MAXINT; ++sweep)
    {
        IRPyro_read_devices(IRPyro_device, num_of_IRPyro_on_demo_board);
        for(int i=0; i<IRPyro_LIST_SIZE; ++i)
        {
            UART_PutString(USART2, (uint8_t*)"SENSOR:");
            UART_PutNumber(USART2, (uint32_t)i);
            UART_PutString(USART2, (uint8_t*)" , ");
            UART_stream_char_send(IRPyro_device[i].channel_value, IRPyro_device[i].channel_saturation_byte);
            // UART_stream_binary_send(IRPyro_device[i].raw_data);
        }
    }
    /*-------------------------- close_list  ---------------------------------*/
    IRPyro_close_devices(IRPyro_device, 4);
    /*-------------------------- collate results -----------------------------*/
}
/**
  * @brief Shows how to change sensor parameters on the run.
  * @attention Select display format to the UART: UART_stream_char_send() or
  *            UART_stream_binary_send() for plain text or binary format package.
  * @note This function demonstrates
    sampling_rate_set,
    sampling_rate_get,
    high_pass_set,
    high_pass_get,
    low_pass_set,
    low_pass_get,
    gain_set,
    gain_get,
    power_off,
    power_normal,
    power_low,
    power_sleep,
    power_wakeup
  */
static void IOCTL_example(void)
{
    /*-------------------------- memory allocation----------------------------*/
    IRPyro_sensor_device_type IRPyro_Sensor =
    {
        .CS_Port  = 2,
        .CS_Pin   = 1,
        .INT_Port = 1,
        .INT_Pin  = 4,
        .address  = 0x75,
        .number_of_active_channels = 1,
        .read_mode=0,
    };
    /*----------------------- --- CONFIGURATION ------------------------------*/
    AFE_reg_type IRPyro_AFE_Single =
    {
        .S7_S0   = 0,                  // Sets the sampling rate to 1,000 samples per second.
        .LP      = LOW_POWER_DISABLE,  // Low power sets the sampling rate to 166 samples per second.
        .HP      = HP_DISABLE,         // High pass filter enabled
        .C_LP    = LOW_PASS_180,       // Low pass filter set at 180Hz
        .CLK_OUT = CLK_ENABLE,         // Clock output enabled to synchronize other IRPyro sensors
        .SYNC    = MASTER,             // IRPyro set as Master issues the synchronization to SLAVE
        .TEMP    = TEMP_DISABLE,       //
        .INT     = INT_ENABLE          // IRPyro indicates data ready enable
    };
    CCP_reg_type IRPyro_Ch_Cfg =
    {
        .Feedback_cap              = FEEDBACK_200_80, // Feedback capacitance
        .High_pass_filter_setting  = HIGH_PASS_1HZ,   // High pass filter set at 1Hz
        .Feedback_transconductance = TCONDUCT_1_2T,   // Time constant selection
        .status_bit                = CH_ENABLE        // Channel ready to collect data
    };
    /*-------------------------- board setup ---------------------------------*/
    Micro_controller_template_init();
    usart_callback_setup();
    // disable all CS lines
    demo_board_sensor_control(0);

    /*-------- Single Sensor Configuration -----------------------------------*/
    IRPyro_Sensor.AFE_register = IRPyro_AFE_Single;
    IRPyro_Sensor.CCP_register[2] =  IRPyro_Ch_Cfg;

    IRPyro_cmd_type IOCTL_cmd = {0};
    IRPyro_arg_type IOCTL_arg = {0};

    IRPyro_open(&IRPyro_Sensor);
    /*------------------------ Sampling rate ---------------------------------*/
    IOCTL_cmd.cmd = sampling_rate_get;  // Normal power, 0x00 = 1ms
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);

    IOCTL_cmd.cmd  = sampling_rate_set; // Normal power, 0xaa = 10 ms
    IOCTL_arg.arg = 0x0A;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);

    IOCTL_cmd.cmd  = sampling_rate_set; // Normal power, 0x00 = 1 ms
    IOCTL_arg.arg = 0x00;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);
    /*------------------------ High pass filter ------------------------------*/
    IOCTL_cmd.cmd = high_pass_set;      // Normal power, 0x02 = 2 Hz
    IOCTL_arg.arg     = HIGH_PASS_2HZ;
    IOCTL_arg.channel = 1;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);

    IOCTL_cmd.cmd = high_pass_get;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);

    IOCTL_cmd.cmd = high_pass_set;
    IOCTL_arg.arg = HIGH_PASS_8HZ;
    IOCTL_arg.channel = 2;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);

    IOCTL_cmd.cmd = high_pass_get;
    read_a_sample(&IRPyro_Sensor);
    /*------------------------ Low pass filter  ------------------------------*/
    IOCTL_cmd.cmd = low_pass_set;
    IOCTL_arg.arg = LOW_PASS_45;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);

    IOCTL_cmd.cmd = low_pass_get;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);
    /*------------------------ Gain changes ----------------------------------*/
    IOCTL_cmd.cmd = gain_set;
    IOCTL_arg.arg = GAIN_08;
    IOCTL_arg.channel = 3;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);

    IOCTL_cmd.cmd = gain_get;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);

    IOCTL_cmd.cmd = gain_set;
    IOCTL_arg.arg = GAIN_16;
    IOCTL_arg.channel = 4;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);
    /*------------------------ Power control ---------------------------------*/
    IOCTL_cmd.cmd = power_normal;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);

    IOCTL_cmd.cmd = power_low;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_Sensor);

    IOCTL_cmd.cmd = power_sleep;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);

    IOCTL_cmd.cmd = power_wakeup;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);

    IOCTL_cmd.cmd = power_off;
    IRPyro_ioctl(&IRPyro_Sensor, &IOCTL_cmd, IOCTL_arg);

    IRPyro_close(&IRPyro_Sensor);
};
/**
 * @brief callback function
 * @attention This function is called from an interrupt service request function
 *            and should be minimal.
 */
static void wakeup_event_callback(void)
{
// add non blocking code
}
/**
 * @brief Wake up event detection
 *        This function configures the IRPyro sensor to sleep mode
 * @attention This function is called from an interrupt service request function
 *            and should be minimal.
 */
static void wake_up_event_detection(void)
{
#define TIME_TO_WAIT 600000
    /*------------------ Memory allocation, initialization-------------------*/
    IRPyro_sensor_device_type IRPyro_single_sensor = {0}; // Instance of sensor
    uint32_t        time_to_wait     = TIME_TO_WAIT;
    uint8_t         wake_up_detected = 0;
    IRPyro_cmd_type IOCTL_cmd        = {0};
    IRPyro_arg_type IOCTL_arg        = {0};

    /*-------------------Physical Location ----------------------------------*/
    IRPyro_single_sensor.CS_Port  = 2;
    IRPyro_single_sensor.CS_Pin   = 1;
    IRPyro_single_sensor.INT_Port = 1;
    IRPyro_single_sensor.INT_Pin  = 4;
    /*-------------------Test configuration ---------------------------------*/
    IRPyro_single_sensor.read_mode = 0;                                       // Interrupt configuration
    IRPyro_single_sensor.address   = 0x08;                                    // I2C address to configure
    IRPyro_single_sensor.number_of_active_channels = 1;                       // configures channel 2 for streaming
    /*------------------- Wake up event definition  -------------------------*/
    IRPyro_single_sensor.WUP_register.UHT          = MAX_WUP_THRESHOLD;       // Upper High Threshold = 255 events
    IRPyro_single_sensor.WUP_register.ULT          = SAFE_WUP_THRESHOLD;      // Upper Low  Threshold = 180 events
    IRPyro_single_sensor.WUP_register.LHT          = MIN_WUP_THRESHOLD;       // Low   High Threshold =   0 events
    IRPyro_single_sensor.WUP_register.LLT          = MIN_WUP_THRESHOLD;       // Low   Low  Threshold =   0 events
    IRPyro_single_sensor.WUP_register.WT_Threshold = DEFAULT_NO_EVENTS;       // Wake up event Threshold = 10 events
    IRPyro_single_sensor.WUP_register.DP0_DP2      = NOT_IN_USE_VALUE;        // Active pixel channel
    IRPyro_single_sensor.WUP_register.CH0_CH2      = SEL_CH_2;                // Reference channel
    IRPyro_single_sensor.WUP_register.ST           = SLEEP_MODE_ONE_CH;       // Single channel selected
    /*-------------------------- board setup ---------------------------------*/
    Micro_controller_template_init();
    usart_callback_setup();
    // disable all CS lines
    demo_board_sensor_control(0);
    /*------------------------ Begin Test ------------------------------------*/
    UART_PutString(USART2, (uint8_t*)"15 Configure sensor WAKE UP EVENT, SEND TO SLEEP, DETECT WAKE UP \n");
    IRPyro_open(&IRPyro_single_sensor);
    UART_PutString(USART2, (uint8_t*)"WAKE UP EVENT DETECTION BY HARDWARE INTERRUPT \n");

    /*------------------------ Send sensor to sleep --------------------------*/
    UART_PutString(USART2, (uint8_t*)"SET IRPyro ON POWER NORMAL \n");
    IOCTL_cmd.cmd = power_normal;
    IRPyro_ioctl(&IRPyro_single_sensor, &IOCTL_cmd, IOCTL_arg);
    read_a_sample(&IRPyro_single_sensor);

    UART_PutString(USART2, (uint8_t*)"SET IRPyro ON POWER SLEEP \n");
    IOCTL_cmd.cmd = power_sleep;
    IRPyro_ioctl(&IRPyro_single_sensor, &IOCTL_cmd, IOCTL_arg);

    /*------------------------ Send MCU to sleep -----------------------------*/
    UART_PutString(USART2, (uint8_t*)"SEND MCU TO SLEEP \n");
    UART_PutString(USART2, (uint8_t*)"WAIT FOR IRPyro TO WAKE MCU \n");
    /*--------------- assign external function to wake-up event --------------*/
    EXTernal_Callback_Setup(wakeup_event_callback);
    // PWR_EnterSTOPMode(PWR_Regulator_LowPower, PWR_STOPEntry_WFI);
    /*--------------- send micro controller to sleep --------------------------*/
    PWR_EnterSleepMode(PWR_SLEEPEntry_WFI);

    /*wait for the sensor to wake up or time to run out*/
    do
    {
        wake_up_detected = EXTI_detected(0);
        --time_to_wait;
    }
    while ((time_to_wait != 0) && (wake_up_detected == 0));

    if (time_to_wait == 0 && wake_up_detected == 0)
    {
        IOCTL_cmd.cmd = power_wakeup;
        UART_PutString(USART2, (uint8_t*)"FORCED POWER WAKE UP \n");
        IRPyro_ioctl(&IRPyro_single_sensor, &IOCTL_cmd, IOCTL_arg);
        UART_PutString(USART2, (uint8_t*)"READ TEST AFTER FORCED POWER WAKE UP \n");
        read_a_sample(&IRPyro_single_sensor);
    }
    else
    {
        UART_PutString(USART2, (uint8_t*)"WAKE UP EVENT DETECTED!! \n");
        UART_PutString(USART2, (uint8_t*)"READ TEST AFTER WAKE UP EVENT \n");
        read_a_sample(&IRPyro_single_sensor);
    }

    IRPyro_close(&IRPyro_single_sensor);
    /*-------------------------- collate results -----------------------------*/
    UART_PutString(USART2, (uint8_t*)"============\n");
    UART_PutString(USART2, (uint8_t*)"--------------WAKE UP EVENT DETECTION BY HARDWARE INTERRUPT-------------------------\n");
}
/**
 * @brief USART transmission function
 *       Code can be added here for the firmware to respond when the transmission software interrupt is enabled
 */
static void tx_function(void)
{

}
/**
 * @brief USART RX function
 *        Code to catch data on the USART interface.
 */
static void rx_function(void)
{
    uint8_t uart_byte = (uint8_t)USART_ReceiveData(USART2);
}
/**
 * @brief configures USART irq functions
 */
static void usart_callback_setup(void)
{
    USART_Callback_Rx_Setup(rx_function);
    USART_Callback_Tx_Setup(tx_function);
}
/**
 * @brief Detect IRPyro sensors present in the demo board
 * @param this_list IRPyro sensor array
 * @param list_size number of sensors in the array
*/
static void IRPyro_sensor_population(IRPyro_devices this_list, uint8_t *list_size)
{
    uint8_t num_of_IRPyro = *list_size;
    uint8_t devices_present =0;
    for(uint8_t current=0 ; current < num_of_IRPyro; ++ current)
    {
        if (IRPyro_open(&this_list[current]) == 1)
        {
            ++ devices_present;
        }
    }
    *list_size = devices_present;
}
/**
 * @brief Changes the state of all the CS lines as used on the IRPyro backplane board
 * @param state   0 disabled / 1 enabled
 */
void demo_board_sensor_control(int state)
{
    CS_pin_set(2,1,state);
    CS_pin_set(2,3,state);
    CS_pin_set(2,4,state);
    CS_pin_set(2,5,state);
}
/** end of IRPyroAPI_Examples
  * @}
  */
/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
