/**
******************************************************************************
* @file    message_generator.c
* @author  _KEMET, Ltd
* @date    March 2018
* @version Release 1.0.6
* @copyright (c) 2018 _KEMET, Ltd
* @brief Recovers requested configuration/data to be send to the GUI
*
* @verbatim
 ===============================================================================
                      ##### Description #####
 ===============================================================================
  [..]
  after the execution of a valid instruction code message_generator()
   - passes the code of packet function
   - sets the next state of eevt: apply parameter or contact_GUI
   - SET case: returns OK /ERR on success or failure
   - GET case: fills a byte buffer with the data requested
  [..]
 @endverbatim
  ******************************************************************************
   @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>
   @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.

    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim

  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#ifndef MESSAGE_TO_GUI
#include "message_generator.h"
#include "version.h"
#endif
#define BYTES_TO_GUI_SIZE (64)
const uint8_t error_string[]    = {0x65,0x72,0x72,0x0A,0x90}; //"err\n";
const uint8_t ok_string[]       = {0x6F,0x6B,0x0A};      //"ok\n";
uint8_t response_required;
uint8_t packet_function;
uint8_t sensor_selected;
uint8_t bytes_to_gui[BYTES_TO_GUI_SIZE];
uint8_t bytes_to_gui_size = 0;
uint8_t JustStartedStreaming ;

/** @defgroup message_interface Message Interface
  * @ingroup eevt_main
  * @brief Defines in firmware each individual command response to GUI commands
  * @verbatim
  *
  * @endverbatim
  * @note
  * @{
  */
/** @defgroup message_generator Message Generator
  * @ingroup message_interface
  * @brief Given a byte string response from the firmware, selects the adequate frame for the GUI.
  * @verbatim
  *
  * @endverbatim
  * @note
  * @{
  */
/**
  * @brief  encodes the AFE register contents of this_device into the IRPyro
  */
static void AFE_to_bytes(AFE_reg_type AFE_register, uint8_t * AFE_byte )
{
    uint8_t bit_value =0;

    AFE_byte[0] = AFE_register.S7_S0;                 // 0b11111111
    bit_value |= (AFE_register.LP      << 7 )& 0X80 ; // 0b10000000
    bit_value |= (AFE_register.HP      << 6 )& 0X40 ; // 0b01000000
    bit_value |= (AFE_register.C_LP    << 4 )& 0X30 ; // 0b00110000
    bit_value |= (AFE_register.CLK_OUT << 3 )& 0X08 ; // 0b00001000
    bit_value |= (AFE_register.SYNC    << 2 )& 0X04 ; // 0b00000100
    bit_value |= (AFE_register.TEMP    << 1 )& 0X02 ; // 0b00000010
    bit_value |= (AFE_register.INT     << 0 )& 0X01 ; // 0b00000001
    AFE_byte[1] = bit_value;
}
/**
  * @brief  encodes the CCP register contents of this_device into IRPyro
  */
static void CCP_to_bytes(CCP_reg_type CCP_register, uint8_t *CCP_byte )
{
    uint8_t bit_value = 0;

    bit_value |= (CCP_register.Feedback_transconductance << 6)& 0XC0 ; // 0b11000000
    bit_value |= (CCP_register.High_pass_filter_setting  << 4)& 0X30 ; // 0b00110000
    bit_value |= (CCP_register.Feedback_cap              << 1)& 0X0E ; // 0b00001110
    bit_value |= (CCP_register.status_bit                << 0)& 0X01 ; // 0b00000001
    *CCP_byte = bit_value;
}
/**
  * @brief  encodes the WUP register contents of this_device into the IRPyro
  */
static void WUP_to_bytes(WUP_reg_type WUP_register, uint8_t * WUP_byte)
{
    uint8_t bit_value = 0;

    WUP_byte[0] = WUP_register.UHT;
    WUP_byte[1] = WUP_register.ULT;
    WUP_byte[2] = WUP_register.LHT;
    WUP_byte[3] = WUP_register.LLT;
    WUP_byte[4] = WUP_register.WT_Threshold;

    bit_value |= (WUP_register.ST      << 6)& 0X40 ; // 0b01000000
    bit_value |= (WUP_register.DP0_DP2 << 3)& 0X38 ; // 0b00111000
    bit_value |= (WUP_register.CH0_CH2 << 0)& 0X07 ; // 0b00000111
    WUP_byte[5] = bit_value;
}
/**
 * @brief initialize bytes array
*/
static void bytes_to_gui_init(void)
{
    memset(bytes_to_gui, 0x00, BYTES_TO_GUI_SIZE);
}
/**
 * @brief calculates the xor on bytes_to_GUI
 * @return xor value
*/
static uint8_t crc_gen(const uint8_t *current_byte, uint8_t bytes_to_process)
{
    uint8_t crc_calc = 0xff; // seed crc calculation
    do
    {
        uint8_t byte_to_crc;
        byte_to_crc = *current_byte;
        crc_calc = crc_calc ^ byte_to_crc;
        ++ current_byte;
        -- bytes_to_process;
    }
    while (bytes_to_process);
    return crc_calc;
}
/**
  * @brief prepares ok string for GUI
  */
static void response_err(void)
{
    error_flag = 0; // reset the error flag
    bytes_to_gui_init();
    bytes_to_gui_size = sizeof(error_string);
    memcpy(bytes_to_gui, error_string, bytes_to_gui_size);
    response_required = 1;
    packet_function = 3;
}
/**
  * @brief prepares ok string for GUI
  */
static void response_ok(void)
{
    uint8_t crc_byte;
    uint8_t ok_size;
    ok_size = sizeof(ok_string);
    bytes_to_gui_init();

    memmove(bytes_to_gui, ok_string, ok_size);
    bytes_to_gui_size = ok_size;
    crc_byte = crc_gen(bytes_to_gui, bytes_to_gui_size);
    memmove(bytes_to_gui + bytes_to_gui_size, &crc_byte, 1);
    ++bytes_to_gui_size;
    response_required = 1;
    packet_function = 3;
}
/**
  * @brief prepares byte array + ok + crc as a byte array for GUI
  */
static void response_byte_array_ok(uint8_t *byte_array, uint8_t size_of_byte_array)
{
    uint8_t crc_byte;
    uint8_t ok_size;
    ok_size = sizeof(ok_string);
    bytes_to_gui_init();

    memcpy(bytes_to_gui, byte_array, size_of_byte_array);
    memmove(bytes_to_gui + size_of_byte_array, ok_string, ok_size);
    bytes_to_gui_size = size_of_byte_array + ok_size;
    crc_byte = crc_gen(bytes_to_gui, bytes_to_gui_size);
    memmove(bytes_to_gui + bytes_to_gui_size, &crc_byte, 1);
    ++bytes_to_gui_size;
    response_required = 1;
    packet_function = 3;
}
/**
  * @brief responds "ok" to GUI
  */
static void null_test_command(void)
{
    response_ok();
}
/**
  * @brief  prepares the firmware version string
  */
static void firmware_version_send(void)
{
#define FIRMWARE_STRING_SIZE 52
    uint8_t msg[FIRMWARE_STRING_SIZE] = "_KEMET API IRPyro messaging application. "VERSION_STRING"\n";
    bytes_to_gui_size = FIRMWARE_STRING_SIZE;
    response_byte_array_ok(msg, bytes_to_gui_size);
    response_required = 1;
    packet_function = 3;
}
/**
  * @brief IRPyro ASIC version byte
  */
static void asic_version_read(void)
{
    uint8_t asic_version_byte[] = {0x21,0x21,0x21,0x21};
    response_byte_array_ok(asic_version_byte, num_of_IRPyro_on_demo_board);
}
/**
  * @brief Get the AFE values
  * Get byte[1] of the AFE register for:
  *   -  specific sensor: indicated by wild card value
  *   -  byte[1] of all the sensors present on the board
  */
static void gen_AFE_read(void)
{
#define AFE_SIZE (2)
    uint8_t afe_bytes[AFE_SIZE];
    uint8_t sensor_selected;
    uint8_t afe_bytes_index = 0;
    uint8_t num_sensors_to_process;
    uint8_t array_of_afe_bytes[NUMBER_IRPyro]= {0};
    uint8_t num_sensors_to_process_for_byte_array;

    process_parms_load(&sensor_selected, &num_sensors_to_process);
    num_sensors_to_process_for_byte_array = num_sensors_to_process;
    do
    {
        AFE_to_bytes(IRPyro_device[sensor_selected].AFE_register,afe_bytes);
        array_of_afe_bytes[afe_bytes_index]=afe_bytes[1];
        ++ afe_bytes_index;
        ++ sensor_selected;
        -- num_sensors_to_process;
    }
    while (num_sensors_to_process);
    response_byte_array_ok(array_of_afe_bytes, num_sensors_to_process_for_byte_array);
}
/**
  * @brief AFE write successful
  */
static void gen_AFE_write(void)
{
    response_ok();
}
/**
  * @brief collects CCP values according to Channel Configuration
  */
static void gen_CCP_read(void)
{
    uint8_t channel_count = 0;
    uint8_t channel_ok = channel_processing(&channel_count);
    if (channel_ok)
    {
        uint8_t array_of_ccp_bytes[NUMBER_OF_SCOPES_IN_THE_GUI] = {0};
        uint8_t channel_count_for_byte_array = channel_count;
        uint8_t byte_array_idx = 0;
        CCP_reg_type CCP_value = {0};
        do
        {
            CCP_value = IRPyro_device[channel_id_to_sensor()].CCP_register[channel_id_to_channel()];

            CCP_to_bytes(CCP_value, &array_of_ccp_bytes[byte_array_idx]);
            channel_id_next();

            ++byte_array_idx;
            -- channel_count;
        }
        while (channel_count);
        channel_count = channel_count_for_byte_array;
        response_byte_array_ok(array_of_ccp_bytes, channel_count);
    }
    else
    {
        response_err();
    }
}
/**
  * @brief CCP write successful
  */
static void gen_CCP_write(void)
{
    response_ok();
}
/**
  * @brief Reads the WUP values of the selected sensor
  */
static void gen_WUP_read(void)
{
#define WUP_SIZE (6)
    uint8_t wup_bytes[WUP_SIZE];
    uint8_t idx;
    uint8_t num_sensors_to_process;
    uint8_t num_sensors_to_process_for_byte_array;
    uint8_t array_of_wup_bytes[NUMBER_IRPyro*WUP_SIZE]= {0};
    uint8_t WUP_offset;

    process_parms_load(&idx, &num_sensors_to_process);
    num_sensors_to_process_for_byte_array = num_sensors_to_process;
    do
    {
        WUP_to_bytes(IRPyro_device[idx].WUP_register,wup_bytes);
        if (num_sensors_to_process_for_byte_array>1)
            WUP_offset = WUP_SIZE * idx;
        else
            WUP_offset = 0;
        for(uint8_t wup_idx =0; wup_idx < WUP_SIZE; ++ wup_idx)
        {
            array_of_wup_bytes[wup_idx + WUP_offset]=wup_bytes[wup_idx];
        }
        ++ idx;
        -- num_sensors_to_process;
    }
    while (num_sensors_to_process);
    response_byte_array_ok(array_of_wup_bytes, (num_sensors_to_process_for_byte_array * WUP_SIZE));
}
/**
  * @brief WUP write successful
  */
static void gen_WUP_write(void)
{
    response_ok();
}
/**
  * @brief streaming start successful
  */
static void streaming_start(void)
{
    JustStartedStreaming = 1;
    response_ok();
}
/**
  * @brief null function to fill unimplemented gaps
  */
static void fndummy(void)
{
    response_required = 0;
}
/**
  * @brief Sleep command successful
  */
static void goto_sleep(void)
{
    IRPyro_device_cmd.cmd = power_wakeup;
    response_ok();
}
/**
  * @brief Wake up command successful
  */
static void wake_up(void)
{
    response_ok();
}
/**
  * @brief Sensor reset successful
  */
static void reset_soft(void)
{
    response_ok();
}
/**
  * @brief board reset successful
  */
static void reset_hard(void)
{
    response_ok();
}
/**
  * @brief hardware reset successful
  */
static void reset_hardware(void)
{
    response_ok();
}

/**
  * @brief returns the value of the sampling rate form the selected sensor
  */
static void sampling_rate_read(void)
{
    uint8_t afe_bytes[2];
    AFE_to_bytes(IRPyro_device[0].AFE_register,afe_bytes);
    response_byte_array_ok(&afe_bytes[0],1);
}

/**
  * @brief Sampling rate operation successful
  */
static uint32_t sample_period_timer;
static void sampling_rate_write(void)
{

    if (IRPyro_device[0].AFE_register.LP)
        sample_period_timer = IRPyro_device[0].AFE_register.S7_S0 * 6;
    else
        sample_period_timer = IRPyro_device[0].AFE_register.S7_S0;

    Micro_controller_sample_period_timer_set(sample_period_timer);
    response_ok();
}
/**
  * @brief returns the value of the current scopes configuration
  */
static void logical_channels_read(void)
{
    response_byte_array_ok(ChannelConfiguration, sizeof(ChannelConfiguration));
}
/**
  * @brief operation successful
  */
static void logical_channels_write(void)
{
    response_ok();
}
/**
  * @brief operation successful
  */
static void set_emitter_timings(void)
{
    response_ok();
}
/**
  * @brief operation successful
  */
static void set_emitter_state(void)
{
    response_ok();
}
/**
  * @brief returns the value of the internal test byte
  */
static void get_failure_flags(void)
{
    response_byte_array_ok(&FailureFlags,1);
}
/**
  * @brief returns the byte identifying the board
  */
static void board_type_read(void)
{
    response_byte_array_ok(&BoardType,1);
}
/**
  * @brief returns the unit id byte
  */
static void unit_id_read(void)
{
    response_byte_array_ok(UnitID,sizeof(UnitID));
}
/**
  * @brief  operation successful
  */
static void unit_id_write(void)
{
    response_ok();
}
/**
  * @brief operation successful
  */
static void second_unit_id_read(void)
{
    response_byte_array_ok(&num_of_IRPyro_on_demo_board,1);
}
/**
  * @brief operation successful
  */
static void get_darkpixel_state(void)
{
    response_byte_array_ok(&DarkPixelState,1);
}
/**
  * @brief operation successful
  */
static void set_darkpixel_state(void)
{
    response_ok();
}
/**
  * @brief operation successful
  */
static void streaming_stop(void)
{
    response_ok();
}
/**
  * @brief Prepare the "err\n" string for the GUI
  */
static void report_error(void)
{
    response_err();
}
/**
  * @brief sends status start completion to GUI
  */
static void status_start(void)
{
    response_ok();
}
/**
  * @brief sends status stop to GUI
  */
static void status_stop(void)
{
    response_ok();
}
/**
 * @brief Places sensor collected data in a buffer according to the map
 * @param scope GUI scope ID
*/
int32_t ConfiguredSensorsSamplingDataToGUI[8] = {0};    // active channel data per device and frame counter
static void data_swap(uint8_t scope)
{
    struct map
    {
        uint8_t sensor_selected;
        uint8_t channel_selected;
    };
    const struct map on_board[] =
    {
        {0,0},
        {0,0},{0,1},{0,2},{0,3},{0,4},
        {1,0},{1,1},{1,2},{1,3},{1,4},
        {2,0},{2,1},{2,2},{2,3},{2,4},
        {3,0},{3,1},{3,2},{3,3},{3,4}
    };

    uint8_t device_selected  = on_board[ChannelConfiguration[scope]].sensor_selected;
    uint8_t channel_selected = on_board[ChannelConfiguration[scope]].channel_selected;
    ConfiguredSensorsSamplingDataToGUI[scope] = IRPyro_device[device_selected].channel_value[channel_selected];
}
/**
 * @brief Sorts sampling data from configured sensors
*/
void message_prepack_data_sort(void)
{
    /*-------------------------- read_list   ---------------------------------*/
    ConfiguredSensorsSamplingDataToGUI[7] = IRPyro_device[0].channel_value[5];
    for(uint8_t scope=0; scope < 7 ; ++ scope)
    {
        if (ChannelConfiguration[scope])
        {
            data_swap(scope);
        }
    }
}
static int32_t sent_sampling_data_frame_cnt    = 0;
static int32_t current_sampling_data_frame_cnt = 1;
/**
  * @brief evaluates conditions to send stream
  */
static bool condition_to_send_stream()
{
    uint8_t time_to_send_sample = 0;

    if ( current_sampling_data_frame_cnt != sent_sampling_data_frame_cnt )
    {
        time_to_send_sample = 1;
    }
    return (isStreaming && time_to_send_sample && !error_flag && !JustStartedStreaming && !status_need_to_send);
}
/**
  * @brief Enables streaming to the GUI
  */
static void message_generator_select_stream(void)
{
    message_prepack_data_sort();
    current_sampling_data_frame_cnt = ConfiguredSensorsSamplingDataToGUI[7];
    if (condition_to_send_stream())
    {
        sent_sampling_data_frame_cnt = current_sampling_data_frame_cnt;
        response_required   = 1;
        packet_function     = 2;
    }
    else
    {
        JustStartedStreaming = 0;
    }
}
/**
  * @brief Send status to GUI on time
  */
static void message_generator_select_status(void)
{
    status_need_to_send = !Micro_controller_timeout_get();
    if (status_need_to_send && isStatus)
    {
        Micro_controller_timeout_set(SEND_STATUS_TIME);
        response_required   = 1;
        packet_function     = 1;
    }
}
/**
  * @brief test the condition to send a packet.
  */
static void message_generator_selector()
{
    message_generator_select_status();
    message_generator_select_stream();
}
/**
 * @brief prepares packet to GUI
*/
void message_generator(uint8_t cmd_to_gui, uint8_t destination)
{
#define CMD_ARRAY_SIZE (35)

    static void (* const pf[CMD_ARRAY_SIZE])(void) =
    {
        //      CODE    CHECK
        null_test_command,           //  0 - 0x00    0xFF
        firmware_version_send,       //  1 - 0x01    0xFE
        asic_version_read,           //  2 - 0x02    0xFD
        gen_CCP_read,                //  3 - 0x03    0xFC
        gen_CCP_write,               //  4 - 0x04    0xFB
        gen_AFE_read,                //  5 - 0x05    0xFA
        gen_AFE_write,               //  6 - 0x06    0xF9
        gen_WUP_read,                //  7 - 0x07    0xF8
        gen_WUP_write,               //  8 - 0x08    0xF7
        streaming_start,             //  9 - 0x09    0xF6
        fndummy,                     // 10 - 0x0A    0xF5 USB_ENG_Read  (DEPRECATED )
        fndummy,                     // 11 - 0x0B    0xF4 USB_ENG_Write (DEPRECATED )
        status_start,                // 12 - 0x0C    0xF3
        status_stop,                 // 13 - 0x0D    0xF2
        goto_sleep,                  // 14 - 0x0E    0xF1
        wake_up,                     // 15 - 0x0F    0xF0
        reset_soft,                  // 16 - 0x10    0xEF
        reset_hard,                  // 17 - 0x11    0xEE
        reset_hardware,              // 18 - 0x12    0xED
        sampling_rate_read,          // 19 - 0x13    0xEC
        sampling_rate_write,         // 20 - 0x14    0xEB
        logical_channels_read,       // 21 - 0x15    0xEA
        logical_channels_write,      // 22 - 0x16    0xE9
        set_emitter_timings,         // 23 - 0x17    0xE8
        set_emitter_state,           // 24 - 0x18    0xE7
        get_failure_flags,           // 25 - 0x19    0xE6
        board_type_read,             // 26 - 0x1A    0xE5
        unit_id_read,                // 27 - 0x1B    0xE4
        unit_id_write,               // 28 - 0x1C    0xE3
        second_unit_id_read,         // 29 - 0x1D    0xE2
        get_darkpixel_state,         // 30 - 0x1E    0xE1
        set_darkpixel_state,         // 31 - 0x1F    0xE0
        streaming_stop,              // 32 - 0x20    0xDF -- Received as code 0x77
        report_error,                // 33 - 0x21    0xDE
        fndummy                      // 34 - 0x22    0xDD
    };
    sensor_selected = destination;
    if (cmd_to_gui < CMD_ARRAY_SIZE)
    {
        (*pf[cmd_to_gui])();
    }
    message_generator_selector();
    if (response_required)
    {
        get_bytes_to_GUI(packet_function, bytes_to_gui, bytes_to_gui_size);
        response_required = 0;
    }
}
/** end of message_generator group
  * @}
  */
/** end of message_interface group
  * @}
  */
/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
